from typing import overload
import typing

import QuantConnect
import QuantConnect.Brokerages
import QuantConnect.Brokerages.GDAX
import QuantConnect.Brokerages.GDAX.Messages
import QuantConnect.Data
import QuantConnect.Data.Market
import QuantConnect.Interfaces
import QuantConnect.Orders
import QuantConnect.Packets
import QuantConnect.Securities
import System
import System.Collections.Concurrent
import System.Collections.Generic


class GDAXFill(System.Object):
    """Tracks fill messages"""

    @property
    def Order(self) -> QuantConnect.Orders.Order:
        """The Lean order"""
        ...

    @property
    def OrderId(self) -> int:
        """Lean orderId"""
        ...

    @property
    def TotalQuantity(self) -> float:
        """Total amount executed across all fills"""
        ...

    @property
    def OrderQuantity(self) -> float:
        """Original order quantity"""
        ...

    def __init__(self, order: QuantConnect.Orders.Order) -> None:
        """Creates instance of GDAXFill"""
        ...

    def Add(self, msg: QuantConnect.Brokerages.GDAX.Messages.Fill) -> None:
        """Adds a trade message"""
        ...


class GDAXBrokerageFactory(QuantConnect.Brokerages.BrokerageFactory):
    """Factory method to create GDAX Websockets brokerage"""

    @property
    def BrokerageData(self) -> System.Collections.Generic.Dictionary[str, str]:
        """provides brokerage connection data"""
        ...

    def __init__(self) -> None:
        """Factory constructor"""
        ...

    def CreateBrokerage(self, job: QuantConnect.Packets.LiveNodePacket, algorithm: QuantConnect.Interfaces.IAlgorithm) -> QuantConnect.Interfaces.IBrokerage:
        """Create the Brokerage instance"""
        ...

    def Dispose(self) -> None:
        """Not required"""
        ...

    def GetBrokerageModel(self, orderProvider: QuantConnect.Securities.IOrderProvider) -> QuantConnect.Brokerages.IBrokerageModel:
        """
        The brokerage model
        
        :param orderProvider: The order provider
        """
        ...


class AuthenticationToken(System.Object):
    """Contains data used for authentication"""

    @property
    def Key(self) -> str:
        """The key"""
        ...

    @Key.setter
    def Key(self, value: str):
        """The key"""
        ...

    @property
    def Signature(self) -> str:
        """The hashed signature"""
        ...

    @Signature.setter
    def Signature(self, value: str):
        """The hashed signature"""
        ...

    @property
    def Timestamp(self) -> str:
        """The timestamp"""
        ...

    @Timestamp.setter
    def Timestamp(self, value: str):
        """The timestamp"""
        ...

    @property
    def Passphrase(self) -> str:
        """The pass phrase"""
        ...

    @Passphrase.setter
    def Passphrase(self, value: str):
        """The pass phrase"""
        ...


class GDAXBrokerage(QuantConnect.Brokerages.BaseWebsocketsBrokerage):
    """Utility methods for GDAX brokerage"""

    SignHeader: str = "CB-ACCESS-SIGN"
    """Sign Header"""

    KeyHeader: str = "CB-ACCESS-KEY"
    """Key Header"""

    TimeHeader: str = "CB-ACCESS-TIMESTAMP"
    """Timestamp Header"""

    PassHeader: str = "CB-ACCESS-PASSPHRASE"
    """Passphrase header"""

    @property
    def IsConnected(self) -> bool:
        ...

    @property
    def FillSplit(self) -> System.Collections.Concurrent.ConcurrentDictionary[int, QuantConnect.Brokerages.GDAX.GDAXFill]:
        ...

    @FillSplit.setter
    def FillSplit(self, value: System.Collections.Concurrent.ConcurrentDictionary[int, QuantConnect.Brokerages.GDAX.GDAXFill]):
        ...

    @property
    def _aggregator(self) -> QuantConnect.Data.IDataAggregator:
        """
        Data Aggregator
        
        This field is protected.
        """
        ...

    @_aggregator.setter
    def _aggregator(self, value: QuantConnect.Data.IDataAggregator):
        """
        Data Aggregator
        
        This field is protected.
        """
        ...

    @property
    def ChannelNames(self) -> typing.List[str]:
        """This property is protected."""
        ...

    @overload
    def __init__(self, name: str) -> None:
        """
        Constructor for brokerage
        
        :param name: Name of brokerage
        """
        ...

    @overload
    def __init__(self, wssUrl: str, websocket: QuantConnect.Brokerages.IWebSocket, restClient: typing.Any, apiKey: str, apiSecret: str, passPhrase: str, algorithm: QuantConnect.Interfaces.IAlgorithm, priceProvider: QuantConnect.Interfaces.IPriceProvider, aggregator: QuantConnect.Data.IDataAggregator, job: QuantConnect.Packets.LiveNodePacket) -> None:
        """
        Constructor for brokerage
        
        :param wssUrl: websockets url
        :param websocket: instance of websockets client
        :param restClient: instance of rest client
        :param apiKey: api key
        :param apiSecret: api secret
        :param passPhrase: pass phrase
        :param algorithm: the algorithm instance is required to retreive account type
        :param priceProvider: The price provider for missing FX conversion rates
        :param aggregator: consolidate ticks
        :param job: The live job packet
        """
        ...

    def CancelOrder(self, order: QuantConnect.Orders.Order) -> bool:
        """Cancels an order"""
        ...

    def Connect(self) -> None:
        """Connects the client to the broker's remote servers"""
        ...

    def Disconnect(self) -> None:
        """Closes the websockets connection"""
        ...

    def Dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def GetAccountHoldings(self) -> System.Collections.Generic.List[QuantConnect.Holding]:
        """Gets all open positions"""
        ...

    @overload
    def GetAuthenticationToken(self, request: typing.Any) -> QuantConnect.Brokerages.GDAX.AuthenticationToken:
        """
        Creates an auth token and adds to the request
        
        :param request: the rest request
        :returns: a token representing the request params.
        """
        ...

    @overload
    def GetAuthenticationToken(self, body: str, method: str, url: str) -> QuantConnect.Brokerages.GDAX.AuthenticationToken:
        """
        Creates an auth token to sign a request
        
        :param body: the request body as json
        :param method: the http method
        :param url: the request url
        """
        ...

    def GetCashBalance(self) -> System.Collections.Generic.List[QuantConnect.Securities.CashAmount]:
        """Gets the total account cash balance"""
        ...

    def GetHistory(self, request: QuantConnect.Data.HistoryRequest) -> System.Collections.Generic.IEnumerable[QuantConnect.Data.BaseData]:
        """
        Gets the history for the requested security
        
        :param request: The historical data request
        :returns: An enumerable of bars covering the span specified in the request.
        """
        ...

    def GetOpenOrders(self) -> System.Collections.Generic.List[QuantConnect.Orders.Order]:
        """Gets all orders not yet closed"""
        ...

    def GetTick(self, symbol: typing.Union[QuantConnect.Symbol, str]) -> QuantConnect.Data.Market.Tick:
        """Retrieves a price tick for a given symbol"""
        ...

    def Initialize(self, wssUrl: str, websocket: QuantConnect.Brokerages.IWebSocket, restClient: typing.Any, apiKey: str, apiSecret: str, passPhrase: str, algorithm: QuantConnect.Interfaces.IAlgorithm, priceProvider: QuantConnect.Interfaces.IPriceProvider, aggregator: QuantConnect.Data.IDataAggregator, job: QuantConnect.Packets.LiveNodePacket) -> None:
        """
        Initialize the instance of this class
        
        This method is protected.
        
        :param wssUrl: The web socket base url
        :param websocket: instance of websockets client
        :param restClient: instance of rest client
        :param apiKey: api key
        :param apiSecret: api secret
        :param passPhrase: pass phrase
        :param algorithm: the algorithm instance is required to retrieve account type
        :param priceProvider: The price provider for missing FX conversion rates
        :param aggregator: the aggregator for consolidating ticks
        :param job: The live job packet
        """
        ...

    def OnMessage(self, sender: typing.Any, webSocketMessage: QuantConnect.Brokerages.WebSocketMessage) -> None:
        """
        Wss message handler
        
        This method is protected.
        """
        ...

    def PlaceOrder(self, order: QuantConnect.Orders.Order) -> bool:
        """Creates a new order"""
        ...

    def PollTick(self, symbol: typing.Union[QuantConnect.Symbol, str]) -> None:
        """Poll for new tick to refresh conversion rate of non-USD denomination"""
        ...

    def Subscribe(self, symbols: System.Collections.Generic.IEnumerable[QuantConnect.Symbol]) -> bool:
        """
        Creates websocket message subscriptions for the supplied symbols
        
        This method is protected.
        """
        ...

    def Unsubscribe(self, symbols: System.Collections.Generic.IEnumerable[QuantConnect.Symbol]) -> bool:
        """Ends current subscriptions"""
        ...

    def UpdateOrder(self, order: QuantConnect.Orders.Order) -> bool:
        """This operation is not supported"""
        ...


class GDAXDataQueueHandler(QuantConnect.Brokerages.GDAX.GDAXBrokerage, QuantConnect.Interfaces.IDataQueueHandler):
    """An implementation of IDataQueueHandler for GDAX"""

    @property
    def ChannelNames(self) -> typing.List[str]:
        """
        The list of websocket channels to subscribe
        
        This property is protected.
        """
        ...

    @overload
    def __init__(self) -> None:
        """Constructor for brokerage"""
        ...

    @overload
    def __init__(self, wssUrl: str, websocket: QuantConnect.Brokerages.IWebSocket, restClient: typing.Any, apiKey: str, apiSecret: str, passPhrase: str, algorithm: QuantConnect.Interfaces.IAlgorithm, priceProvider: QuantConnect.Interfaces.IPriceProvider, aggregator: QuantConnect.Data.IDataAggregator, job: QuantConnect.Packets.LiveNodePacket) -> None:
        """Initializes a new instance of the GDAXDataQueueHandler class"""
        ...

    def SetJob(self, job: QuantConnect.Packets.LiveNodePacket) -> None:
        """
        Sets the job we're subscribing for
        
        :param job: Job we're subscribing for
        """
        ...

    def Subscribe(self, dataConfig: QuantConnect.Data.SubscriptionDataConfig, newDataAvailableHandler: typing.Callable[[System.Object, System.EventArgs], None]) -> System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]:
        """
        Subscribe to the specified configuration
        
        :param dataConfig: defines the parameters to subscribe to a data feed
        :param newDataAvailableHandler: handler to be fired on new data available
        :returns: The new enumerator for this subscription request.
        """
        ...

    def Unsubscribe(self, dataConfig: QuantConnect.Data.SubscriptionDataConfig) -> None:
        """
        Removes the specified configuration
        
        :param dataConfig: Subscription config to be removed
        """
        ...





from typing import overload
import typing

import QuantConnect
import QuantConnect.Data
import QuantConnect.Securities
import QuantConnect.Securities.Index
import System


class IndexCache(QuantConnect.Securities.SecurityCache):
    """INDEX specific caching support"""


class IndexDataFilter(QuantConnect.Securities.SecurityDataFilter):
    """Index packet by packet data filtering mechanism for dynamically detecting bad ticks."""


class IndexExchange(QuantConnect.Securities.SecurityExchange):
    """INDEX exchange class - information and helper tools for Index exchange properties"""

    @property
    def TradingDaysPerYear(self) -> int:
        """Number of trading days per year for this security, used for performance statistics."""
        ...

    def __init__(self, exchangeHours: QuantConnect.Securities.SecurityExchangeHours) -> None:
        """
        Initializes a new instance of the IndexExchange class using the specified
        exchange hours to determine open/close times
        
        :param exchangeHours: Contains the weekly exchange schedule plus holidays
        """
        ...


class IndexSymbol(System.Object):
    """Helper methods for Index Symbols"""

    @staticmethod
    def GetIndexExchange(symbol: typing.Union[QuantConnect.Symbol, str]) -> str:
        """
        Gets the actual exchange the index lives on
        
        :returns: The market of the index.
        """
        ...


class Index(QuantConnect.Securities.Security):
    """INDEX Security Object Implementation for INDEX Assets"""

    @property
    def IsTradable(self) -> bool:
        """Gets or sets whether or not this security should be considered tradable"""
        ...

    @IsTradable.setter
    def IsTradable(self, value: bool):
        """Gets or sets whether or not this security should be considered tradable"""
        ...

    @overload
    def __init__(self, exchangeHours: QuantConnect.Securities.SecurityExchangeHours, quoteCurrency: QuantConnect.Securities.Cash, config: QuantConnect.Data.SubscriptionDataConfig, symbolProperties: QuantConnect.Securities.SymbolProperties, currencyConverter: QuantConnect.Securities.ICurrencyConverter, registeredTypes: QuantConnect.Securities.IRegisteredSecurityDataTypesProvider) -> None:
        """
        Constructor for the INDEX security
        
        :param exchangeHours: Defines the hours this exchange is open
        :param quoteCurrency: The cash object that represent the quote currency
        :param config: The subscription configuration for this security
        :param symbolProperties: The symbol properties for this security
        :param currencyConverter: Currency converter used to convert CashAmount instances into units of the account currency
        :param registeredTypes: Provides all data types registered in the algorithm
        """
        ...

    @overload
    def __init__(self, symbol: typing.Union[QuantConnect.Symbol, str], exchangeHours: QuantConnect.Securities.SecurityExchangeHours, quoteCurrency: QuantConnect.Securities.Cash, symbolProperties: QuantConnect.Securities.SymbolProperties, currencyConverter: QuantConnect.Securities.ICurrencyConverter, registeredTypes: QuantConnect.Securities.IRegisteredSecurityDataTypesProvider, securityCache: QuantConnect.Securities.SecurityCache) -> None:
        """
        Constructor for the INDEX security
        
        :param symbol: The security's symbol
        :param exchangeHours: Defines the hours this exchange is open
        :param quoteCurrency: The cash object that represent the quote currency
        :param symbolProperties: The symbol properties for this security
        :param currencyConverter: Currency converter used to convert CashAmount instances into units of the account currency
        :param registeredTypes: Provides all data types registered in the algorithm
        :param securityCache: Cache to store security information
        """
        ...


class IndexHolding(QuantConnect.Securities.SecurityHolding):
    """Index holdings implementation of the base securities class"""

    def __init__(self, security: QuantConnect.Securities.Index.Index, currencyConverter: QuantConnect.Securities.ICurrencyConverter) -> None:
        """
        INDEX Holding Class constructor
        
        :param security: The INDEX security being held
        :param currencyConverter: A currency converter instance
        """
        ...





from typing import overload
import abc
import typing

import System
import System.Collections
import System.Collections.Generic
import System.Collections.ObjectModel
import System.Collections.Specialized
import System.ComponentModel

System_Collections_ObjectModel_Collection_T = typing.TypeVar("System_Collections_ObjectModel_Collection_T")
System_Collections_ObjectModel_ReadOnlyDictionary_TKey = typing.TypeVar("System_Collections_ObjectModel_ReadOnlyDictionary_TKey")
System_Collections_ObjectModel_ReadOnlyDictionary_TValue = typing.TypeVar("System_Collections_ObjectModel_ReadOnlyDictionary_TValue")
System_Collections_ObjectModel_ReadOnlyCollection_T = typing.TypeVar("System_Collections_ObjectModel_ReadOnlyCollection_T")
System_Collections_ObjectModel_ObservableCollection_T = typing.TypeVar("System_Collections_ObjectModel_ObservableCollection_T")
System_Collections_ObjectModel_ReadOnlyObservableCollection_T = typing.TypeVar("System_Collections_ObjectModel_ReadOnlyObservableCollection_T")
System_Collections_ObjectModel_KeyedCollection_TItem = typing.TypeVar("System_Collections_ObjectModel_KeyedCollection_TItem")
System_Collections_ObjectModel_KeyedCollection_TKey = typing.TypeVar("System_Collections_ObjectModel_KeyedCollection_TKey")
System_Collections_ObjectModel__EventContainer_Callable = typing.TypeVar("System_Collections_ObjectModel__EventContainer_Callable")
System_Collections_ObjectModel__EventContainer_ReturnType = typing.TypeVar("System_Collections_ObjectModel__EventContainer_ReturnType")


class Collection(typing.Generic[System_Collections_ObjectModel_Collection_T], System.Object, System.Collections.Generic.IList[System_Collections_ObjectModel_Collection_T], System.Collections.IList, System.Collections.Generic.IReadOnlyList[System_Collections_ObjectModel_Collection_T], typing.Iterable[System_Collections_ObjectModel_Collection_T]):
    """This class has no documentation."""

    @property
    def Count(self) -> int:
        ...

    @property
    def Items(self) -> System.Collections.Generic.IList[System_Collections_ObjectModel_Collection_T]:
        """This property is protected."""
        ...

    @property
    def IsReadOnly(self) -> bool:
        ...

    @property
    def IsSynchronized(self) -> bool:
        ...

    @property
    def SyncRoot(self) -> System.Object:
        ...

    @property
    def IsFixedSize(self) -> bool:
        ...

    @overload
    def __getitem__(self, index: int) -> System_Collections_ObjectModel_Collection_T:
        ...

    @overload
    def __getitem__(self, index: int) -> typing.Any:
        ...

    @overload
    def __init__(self) -> None:
        ...

    @overload
    def __init__(self, list: System.Collections.Generic.IList[System_Collections_ObjectModel_Collection_T]) -> None:
        ...

    @overload
    def __setitem__(self, index: int, value: System_Collections_ObjectModel_Collection_T) -> None:
        ...

    @overload
    def __setitem__(self, index: int, value: typing.Any) -> None:
        ...

    @overload
    def Add(self, item: System_Collections_ObjectModel_Collection_T) -> None:
        ...

    @overload
    def Add(self, value: typing.Any) -> int:
        ...

    def Clear(self) -> None:
        ...

    def ClearItems(self) -> None:
        """This method is protected."""
        ...

    @overload
    def Contains(self, item: System_Collections_ObjectModel_Collection_T) -> bool:
        ...

    @overload
    def Contains(self, value: typing.Any) -> bool:
        ...

    @overload
    def CopyTo(self, array: typing.List[System_Collections_ObjectModel_Collection_T], index: int) -> None:
        ...

    @overload
    def CopyTo(self, array: System.Array, index: int) -> None:
        ...

    @overload
    def GetEnumerator(self) -> System.Collections.Generic.IEnumerator[System_Collections_ObjectModel_Collection_T]:
        ...

    @overload
    def GetEnumerator(self) -> System.Collections.IEnumerator:
        ...

    @overload
    def IndexOf(self, item: System_Collections_ObjectModel_Collection_T) -> int:
        ...

    @overload
    def IndexOf(self, value: typing.Any) -> int:
        ...

    @overload
    def Insert(self, index: int, item: System_Collections_ObjectModel_Collection_T) -> None:
        ...

    @overload
    def Insert(self, index: int, value: typing.Any) -> None:
        ...

    def InsertItem(self, index: int, item: System_Collections_ObjectModel_Collection_T) -> None:
        """This method is protected."""
        ...

    @overload
    def Remove(self, item: System_Collections_ObjectModel_Collection_T) -> bool:
        ...

    @overload
    def Remove(self, value: typing.Any) -> None:
        ...

    def RemoveAt(self, index: int) -> None:
        ...

    def RemoveItem(self, index: int) -> None:
        """This method is protected."""
        ...

    def SetItem(self, index: int, item: System_Collections_ObjectModel_Collection_T) -> None:
        """This method is protected."""
        ...


class ReadOnlyDictionary(typing.Generic[System_Collections_ObjectModel_ReadOnlyDictionary_TKey, System_Collections_ObjectModel_ReadOnlyDictionary_TValue], System.Object, System.Collections.Generic.IDictionary[System_Collections_ObjectModel_ReadOnlyDictionary_TKey, System_Collections_ObjectModel_ReadOnlyDictionary_TValue], System.Collections.IDictionary, System.Collections.Generic.IReadOnlyDictionary[System_Collections_ObjectModel_ReadOnlyDictionary_TKey, System_Collections_ObjectModel_ReadOnlyDictionary_TValue], typing.Iterable[System.Collections.Generic.KeyValuePair[System_Collections_ObjectModel_ReadOnlyDictionary_TKey, System_Collections_ObjectModel_ReadOnlyDictionary_TValue]]):
    """This class has no documentation."""

    class KeyCollection(System.Object, System.Collections.Generic.ICollection[System_Collections_ObjectModel_ReadOnlyDictionary_TKey], System.Collections.ICollection, System.Collections.Generic.IReadOnlyCollection[System_Collections_ObjectModel_ReadOnlyDictionary_TKey], typing.Iterable[System_Collections_ObjectModel_ReadOnlyDictionary_TKey]):
        """This class has no documentation."""

        @property
        def Count(self) -> int:
            ...

        @property
        def IsReadOnly(self) -> bool:
            ...

        @property
        def IsSynchronized(self) -> bool:
            ...

        @property
        def SyncRoot(self) -> System.Object:
            ...

        def Add(self, item: System_Collections_ObjectModel_ReadOnlyDictionary_TKey) -> None:
            ...

        def Clear(self) -> None:
            ...

        def Contains(self, item: System_Collections_ObjectModel_ReadOnlyDictionary_TKey) -> bool:
            ...

        @overload
        def CopyTo(self, array: typing.List[System_Collections_ObjectModel_ReadOnlyDictionary_TKey], arrayIndex: int) -> None:
            ...

        @overload
        def CopyTo(self, array: System.Array, index: int) -> None:
            ...

        @overload
        def GetEnumerator(self) -> System.Collections.Generic.IEnumerator[System_Collections_ObjectModel_ReadOnlyDictionary_TKey]:
            ...

        @overload
        def GetEnumerator(self) -> System.Collections.IEnumerator:
            ...

        def Remove(self, item: System_Collections_ObjectModel_ReadOnlyDictionary_TKey) -> bool:
            ...

    class ValueCollection(System.Object, System.Collections.Generic.ICollection[System_Collections_ObjectModel_ReadOnlyDictionary_TValue], System.Collections.ICollection, System.Collections.Generic.IReadOnlyCollection[System_Collections_ObjectModel_ReadOnlyDictionary_TValue], typing.Iterable[System_Collections_ObjectModel_ReadOnlyDictionary_TValue]):
        """This class has no documentation."""

        @property
        def Count(self) -> int:
            ...

        @property
        def IsReadOnly(self) -> bool:
            ...

        @property
        def IsSynchronized(self) -> bool:
            ...

        @property
        def SyncRoot(self) -> System.Object:
            ...

        def Add(self, item: System_Collections_ObjectModel_ReadOnlyDictionary_TValue) -> None:
            ...

        def Clear(self) -> None:
            ...

        def Contains(self, item: System_Collections_ObjectModel_ReadOnlyDictionary_TValue) -> bool:
            ...

        @overload
        def CopyTo(self, array: typing.List[System_Collections_ObjectModel_ReadOnlyDictionary_TValue], arrayIndex: int) -> None:
            ...

        @overload
        def CopyTo(self, array: System.Array, index: int) -> None:
            ...

        @overload
        def GetEnumerator(self) -> System.Collections.Generic.IEnumerator[System_Collections_ObjectModel_ReadOnlyDictionary_TValue]:
            ...

        @overload
        def GetEnumerator(self) -> System.Collections.IEnumerator:
            ...

        def Remove(self, item: System_Collections_ObjectModel_ReadOnlyDictionary_TValue) -> bool:
            ...

    @property
    def Dictionary(self) -> System.Collections.Generic.IDictionary[System_Collections_ObjectModel_ReadOnlyDictionary_TKey, System_Collections_ObjectModel_ReadOnlyDictionary_TValue]:
        """This property is protected."""
        ...

    @property
    def Keys(self) -> System.Collections.ObjectModel.ReadOnlyDictionary.KeyCollection:
        ...

    @property
    def Values(self) -> System.Collections.ObjectModel.ReadOnlyDictionary.ValueCollection:
        ...

    @property
    def Count(self) -> int:
        ...

    @property
    def IsReadOnly(self) -> bool:
        ...

    @property
    def IsFixedSize(self) -> bool:
        ...

    @property
    def IsSynchronized(self) -> bool:
        ...

    @property
    def SyncRoot(self) -> System.Object:
        ...

    @overload
    def __getitem__(self, key: System_Collections_ObjectModel_ReadOnlyDictionary_TKey) -> System_Collections_ObjectModel_ReadOnlyDictionary_TValue:
        ...

    @overload
    def __getitem__(self, key: System_Collections_ObjectModel_ReadOnlyDictionary_TKey) -> System_Collections_ObjectModel_ReadOnlyDictionary_TValue:
        ...

    @overload
    def __getitem__(self, key: typing.Any) -> typing.Any:
        ...

    def __init__(self, dictionary: System.Collections.Generic.IDictionary[System_Collections_ObjectModel_ReadOnlyDictionary_TKey, System_Collections_ObjectModel_ReadOnlyDictionary_TValue]) -> None:
        ...

    @overload
    def __setitem__(self, key: System_Collections_ObjectModel_ReadOnlyDictionary_TKey, value: System_Collections_ObjectModel_ReadOnlyDictionary_TValue) -> None:
        ...

    @overload
    def __setitem__(self, key: typing.Any, value: typing.Any) -> None:
        ...

    @overload
    def Add(self, key: System_Collections_ObjectModel_ReadOnlyDictionary_TKey, value: System_Collections_ObjectModel_ReadOnlyDictionary_TValue) -> None:
        ...

    @overload
    def Add(self, item: System.Collections.Generic.KeyValuePair[System_Collections_ObjectModel_ReadOnlyDictionary_TKey, System_Collections_ObjectModel_ReadOnlyDictionary_TValue]) -> None:
        ...

    @overload
    def Add(self, key: typing.Any, value: typing.Any) -> None:
        ...

    @overload
    def Clear(self) -> None:
        ...

    @overload
    def Clear(self) -> None:
        ...

    @overload
    def Contains(self, item: System.Collections.Generic.KeyValuePair[System_Collections_ObjectModel_ReadOnlyDictionary_TKey, System_Collections_ObjectModel_ReadOnlyDictionary_TValue]) -> bool:
        ...

    @overload
    def Contains(self, key: typing.Any) -> bool:
        ...

    def ContainsKey(self, key: System_Collections_ObjectModel_ReadOnlyDictionary_TKey) -> bool:
        ...

    @overload
    def CopyTo(self, array: typing.List[System.Collections.Generic.KeyValuePair[System_Collections_ObjectModel_ReadOnlyDictionary_TKey, System_Collections_ObjectModel_ReadOnlyDictionary_TValue]], arrayIndex: int) -> None:
        ...

    @overload
    def CopyTo(self, array: System.Array, index: int) -> None:
        ...

    @overload
    def GetEnumerator(self) -> System.Collections.Generic.IEnumerator[System.Collections.Generic.KeyValuePair[System_Collections_ObjectModel_ReadOnlyDictionary_TKey, System_Collections_ObjectModel_ReadOnlyDictionary_TValue]]:
        ...

    @overload
    def GetEnumerator(self) -> System.Collections.IEnumerator:
        ...

    @overload
    def GetEnumerator(self) -> System.Collections.IDictionaryEnumerator:
        ...

    @overload
    def Remove(self, key: System_Collections_ObjectModel_ReadOnlyDictionary_TKey) -> bool:
        ...

    @overload
    def Remove(self, item: System.Collections.Generic.KeyValuePair[System_Collections_ObjectModel_ReadOnlyDictionary_TKey, System_Collections_ObjectModel_ReadOnlyDictionary_TValue]) -> bool:
        ...

    @overload
    def Remove(self, key: typing.Any) -> None:
        ...

    def TryGetValue(self, key: System_Collections_ObjectModel_ReadOnlyDictionary_TKey, value: typing.Optional[System_Collections_ObjectModel_ReadOnlyDictionary_TValue]) -> typing.Union[bool, System_Collections_ObjectModel_ReadOnlyDictionary_TValue]:
        ...


class ReadOnlyCollection(typing.Generic[System_Collections_ObjectModel_ReadOnlyCollection_T], System.Object, System.Collections.Generic.IList[System_Collections_ObjectModel_ReadOnlyCollection_T], System.Collections.IList, System.Collections.Generic.IReadOnlyList[System_Collections_ObjectModel_ReadOnlyCollection_T], typing.Iterable[System_Collections_ObjectModel_ReadOnlyCollection_T]):
    """This class has no documentation."""

    @property
    def Count(self) -> int:
        ...

    @property
    def Items(self) -> System.Collections.Generic.IList[System_Collections_ObjectModel_ReadOnlyCollection_T]:
        """This property is protected."""
        ...

    @property
    def IsReadOnly(self) -> bool:
        ...

    @property
    def IsSynchronized(self) -> bool:
        ...

    @property
    def SyncRoot(self) -> System.Object:
        ...

    @property
    def IsFixedSize(self) -> bool:
        ...

    @overload
    def __getitem__(self, index: int) -> System_Collections_ObjectModel_ReadOnlyCollection_T:
        ...

    @overload
    def __getitem__(self, index: int) -> System_Collections_ObjectModel_ReadOnlyCollection_T:
        ...

    @overload
    def __getitem__(self, index: int) -> typing.Any:
        ...

    def __init__(self, list: System.Collections.Generic.IList[System_Collections_ObjectModel_ReadOnlyCollection_T]) -> None:
        ...

    @overload
    def __setitem__(self, index: int, value: System_Collections_ObjectModel_ReadOnlyCollection_T) -> None:
        ...

    @overload
    def __setitem__(self, index: int, value: typing.Any) -> None:
        ...

    @overload
    def Add(self, value: System_Collections_ObjectModel_ReadOnlyCollection_T) -> None:
        ...

    @overload
    def Add(self, value: typing.Any) -> int:
        ...

    @overload
    def Clear(self) -> None:
        ...

    @overload
    def Clear(self) -> None:
        ...

    @overload
    def Contains(self, value: System_Collections_ObjectModel_ReadOnlyCollection_T) -> bool:
        ...

    @overload
    def Contains(self, value: typing.Any) -> bool:
        ...

    @overload
    def CopyTo(self, array: typing.List[System_Collections_ObjectModel_ReadOnlyCollection_T], index: int) -> None:
        ...

    @overload
    def CopyTo(self, array: System.Array, index: int) -> None:
        ...

    @overload
    def GetEnumerator(self) -> System.Collections.Generic.IEnumerator[System_Collections_ObjectModel_ReadOnlyCollection_T]:
        ...

    @overload
    def GetEnumerator(self) -> System.Collections.IEnumerator:
        ...

    @overload
    def IndexOf(self, value: System_Collections_ObjectModel_ReadOnlyCollection_T) -> int:
        ...

    @overload
    def IndexOf(self, value: typing.Any) -> int:
        ...

    @overload
    def Insert(self, index: int, value: System_Collections_ObjectModel_ReadOnlyCollection_T) -> None:
        ...

    @overload
    def Insert(self, index: int, value: typing.Any) -> None:
        ...

    @overload
    def Remove(self, value: System_Collections_ObjectModel_ReadOnlyCollection_T) -> bool:
        ...

    @overload
    def Remove(self, value: typing.Any) -> None:
        ...

    @overload
    def RemoveAt(self, index: int) -> None:
        ...

    @overload
    def RemoveAt(self, index: int) -> None:
        ...


class ObservableCollection(typing.Generic[System_Collections_ObjectModel_ObservableCollection_T], System.Collections.ObjectModel.Collection[System_Collections_ObjectModel_ObservableCollection_T], System.Collections.Specialized.INotifyCollectionChanged, System.ComponentModel.INotifyPropertyChanged):
    """
    Implementation of a dynamic data collection based on generic Collection<T>,
    implementing INotifyCollectionChanged to notify listeners
    when items get added, removed or the whole list is refreshed.
    """

    @property
    def PropertyChanged(self) -> _EventContainer[typing.Callable[[System.Object, System.ComponentModel.PropertyChangedEventArgs], None], None]:
        """PropertyChanged event (per INotifyPropertyChanged)."""
        ...

    @PropertyChanged.setter
    def PropertyChanged(self, value: _EventContainer[typing.Callable[[System.Object, System.ComponentModel.PropertyChangedEventArgs], None], None]):
        """PropertyChanged event (per INotifyPropertyChanged)."""
        ...

    @property
    def CollectionChanged(self) -> _EventContainer[typing.Callable[[System.Object, System.Collections.Specialized.NotifyCollectionChangedEventArgs], None], None]:
        """Occurs when the collection changes, either by adding or removing an item."""
        ...

    @CollectionChanged.setter
    def CollectionChanged(self, value: _EventContainer[typing.Callable[[System.Object, System.Collections.Specialized.NotifyCollectionChangedEventArgs], None], None]):
        """Occurs when the collection changes, either by adding or removing an item."""
        ...

    @property
    def PropertyChanged(self) -> _EventContainer[typing.Callable[[System.Object, System.ComponentModel.PropertyChangedEventArgs], None], None]:
        """
        PropertyChanged event (per INotifyPropertyChanged).
        
        This field is protected.
        """
        ...

    @PropertyChanged.setter
    def PropertyChanged(self, value: _EventContainer[typing.Callable[[System.Object, System.ComponentModel.PropertyChangedEventArgs], None], None]):
        """
        PropertyChanged event (per INotifyPropertyChanged).
        
        This field is protected.
        """
        ...

    @overload
    def __init__(self) -> None:
        """Initializes a new instance of ObservableCollection that is empty and has default initial capacity."""
        ...

    @overload
    def __init__(self, collection: System.Collections.Generic.IEnumerable[System_Collections_ObjectModel_ObservableCollection_T]) -> None:
        """
        Initializes a new instance of the ObservableCollection class that contains
        elements copied from the specified collection and has sufficient capacity
        to accommodate the number of elements copied.
        
        :param collection: The collection whose elements are copied to the new list.
        """
        ...

    @overload
    def __init__(self, list: System.Collections.Generic.List[System_Collections_ObjectModel_ObservableCollection_T]) -> None:
        """
        Initializes a new instance of the ObservableCollection class
        that contains elements copied from the specified list
        
        :param list: The list whose elements are copied to the new list.
        """
        ...

    def BlockReentrancy(self) -> System.IDisposable:
        """
        Disallow reentrant attempts to change this collection. E.g. an event handler
        of the CollectionChanged event is not allowed to make changes to this collection.
        
        This method is protected.
        """
        ...

    def CheckReentrancy(self) -> None:
        """
        Check and assert for reentrant attempts to change this collection.
        
        This method is protected.
        """
        ...

    def ClearItems(self) -> None:
        """
        Called by base class Collection<T> when the list is being cleared;
        raises a CollectionChanged event to any listeners.
        
        This method is protected.
        """
        ...

    def InsertItem(self, index: int, item: System_Collections_ObjectModel_ObservableCollection_T) -> None:
        """
        Called by base class Collection<T> when an item is added to list;
        raises a CollectionChanged event to any listeners.
        
        This method is protected.
        """
        ...

    def Move(self, oldIndex: int, newIndex: int) -> None:
        """Move item at oldIndex to newIndex."""
        ...

    def MoveItem(self, oldIndex: int, newIndex: int) -> None:
        """
        Called by base class ObservableCollection<T> when an item is to be moved within the list;
        raises a CollectionChanged event to any listeners.
        
        This method is protected.
        """
        ...

    def OnCollectionChanged(self, e: System.Collections.Specialized.NotifyCollectionChangedEventArgs) -> None:
        """
        Raise CollectionChanged event to any listeners.
        Properties/methods modifying this ObservableCollection will raise
        a collection changed event through this virtual method.
        
        This method is protected.
        """
        ...

    def OnPropertyChanged(self, e: System.ComponentModel.PropertyChangedEventArgs) -> None:
        """
        Raises a PropertyChanged event (per INotifyPropertyChanged).
        
        This method is protected.
        """
        ...

    def RemoveItem(self, index: int) -> None:
        """
        Called by base class Collection<T> when an item is removed from list;
        raises a CollectionChanged event to any listeners.
        
        This method is protected.
        """
        ...

    def SetItem(self, index: int, item: System_Collections_ObjectModel_ObservableCollection_T) -> None:
        """
        Called by base class Collection<T> when an item is set in list;
        raises a CollectionChanged event to any listeners.
        
        This method is protected.
        """
        ...


class ReadOnlyObservableCollection(typing.Generic[System_Collections_ObjectModel_ReadOnlyObservableCollection_T], System.Collections.ObjectModel.ReadOnlyCollection[System_Collections_ObjectModel_ReadOnlyObservableCollection_T], System.Collections.Specialized.INotifyCollectionChanged, System.ComponentModel.INotifyPropertyChanged):
    """Read-only wrapper around an ObservableCollection."""

    @property
    def CollectionChanged(self) -> _EventContainer[typing.Callable[[System.Object, System.Collections.Specialized.NotifyCollectionChangedEventArgs], None], None]:
        """CollectionChanged event (per INotifyCollectionChanged)."""
        ...

    @CollectionChanged.setter
    def CollectionChanged(self, value: _EventContainer[typing.Callable[[System.Object, System.Collections.Specialized.NotifyCollectionChangedEventArgs], None], None]):
        """CollectionChanged event (per INotifyCollectionChanged)."""
        ...

    @property
    def CollectionChanged(self) -> _EventContainer[typing.Callable[[System.Object, System.Collections.Specialized.NotifyCollectionChangedEventArgs], None], None]:
        """
        Occurs when the collection changes, either by adding or removing an item.
        
        This field is protected.
        """
        ...

    @CollectionChanged.setter
    def CollectionChanged(self, value: _EventContainer[typing.Callable[[System.Object, System.Collections.Specialized.NotifyCollectionChangedEventArgs], None], None]):
        """
        Occurs when the collection changes, either by adding or removing an item.
        
        This field is protected.
        """
        ...

    @property
    def PropertyChanged(self) -> _EventContainer[typing.Callable[[System.Object, System.ComponentModel.PropertyChangedEventArgs], None], None]:
        """PropertyChanged event (per INotifyPropertyChanged)."""
        ...

    @PropertyChanged.setter
    def PropertyChanged(self, value: _EventContainer[typing.Callable[[System.Object, System.ComponentModel.PropertyChangedEventArgs], None], None]):
        """PropertyChanged event (per INotifyPropertyChanged)."""
        ...

    @property
    def PropertyChanged(self) -> _EventContainer[typing.Callable[[System.Object, System.ComponentModel.PropertyChangedEventArgs], None], None]:
        """
        Occurs when a property changes.
        
        This field is protected.
        """
        ...

    @PropertyChanged.setter
    def PropertyChanged(self, value: _EventContainer[typing.Callable[[System.Object, System.ComponentModel.PropertyChangedEventArgs], None], None]):
        """
        Occurs when a property changes.
        
        This field is protected.
        """
        ...

    def __init__(self, list: System.Collections.ObjectModel.ObservableCollection[System_Collections_ObjectModel_ReadOnlyObservableCollection_T]) -> None:
        """
        Initializes a new instance of ReadOnlyObservableCollection that
        wraps the given ObservableCollection.
        """
        ...

    def OnCollectionChanged(self, args: System.Collections.Specialized.NotifyCollectionChangedEventArgs) -> None:
        """
        raise CollectionChanged event to any listeners
        
        This method is protected.
        """
        ...

    def OnPropertyChanged(self, args: System.ComponentModel.PropertyChangedEventArgs) -> None:
        """
        raise PropertyChanged event to any listeners
        
        This method is protected.
        """
        ...


class KeyedCollection(typing.Generic[System_Collections_ObjectModel_KeyedCollection_TKey, System_Collections_ObjectModel_KeyedCollection_TItem], System.Collections.ObjectModel.Collection[System_Collections_ObjectModel_KeyedCollection_TItem], metaclass=abc.ABCMeta):
    """This class has no documentation."""

    @property
    def Comparer(self) -> System.Collections.Generic.IEqualityComparer[System_Collections_ObjectModel_KeyedCollection_TKey]:
        ...

    @property
    def Dictionary(self) -> System.Collections.Generic.IDictionary[System_Collections_ObjectModel_KeyedCollection_TKey, System_Collections_ObjectModel_KeyedCollection_TItem]:
        """This property is protected."""
        ...

    def __getitem__(self, key: System_Collections_ObjectModel_KeyedCollection_TKey) -> System_Collections_ObjectModel_KeyedCollection_TItem:
        ...

    @overload
    def __init__(self) -> None:
        """This method is protected."""
        ...

    @overload
    def __init__(self, comparer: System.Collections.Generic.IEqualityComparer[System_Collections_ObjectModel_KeyedCollection_TKey]) -> None:
        """This method is protected."""
        ...

    @overload
    def __init__(self, comparer: System.Collections.Generic.IEqualityComparer[System_Collections_ObjectModel_KeyedCollection_TKey], dictionaryCreationThreshold: int) -> None:
        """This method is protected."""
        ...

    def ChangeItemKey(self, item: System_Collections_ObjectModel_KeyedCollection_TItem, newKey: System_Collections_ObjectModel_KeyedCollection_TKey) -> None:
        """This method is protected."""
        ...

    def ClearItems(self) -> None:
        """This method is protected."""
        ...

    def Contains(self, key: System_Collections_ObjectModel_KeyedCollection_TKey) -> bool:
        ...

    def GetKeyForItem(self, item: System_Collections_ObjectModel_KeyedCollection_TItem) -> System_Collections_ObjectModel_KeyedCollection_TKey:
        """This method is protected."""
        ...

    def InsertItem(self, index: int, item: System_Collections_ObjectModel_KeyedCollection_TItem) -> None:
        """This method is protected."""
        ...

    def Remove(self, key: System_Collections_ObjectModel_KeyedCollection_TKey) -> bool:
        ...

    def RemoveItem(self, index: int) -> None:
        """This method is protected."""
        ...

    def SetItem(self, index: int, item: System_Collections_ObjectModel_KeyedCollection_TItem) -> None:
        """This method is protected."""
        ...

    def TryGetValue(self, key: System_Collections_ObjectModel_KeyedCollection_TKey, item: typing.Optional[System_Collections_ObjectModel_KeyedCollection_TItem]) -> typing.Union[bool, System_Collections_ObjectModel_KeyedCollection_TItem]:
        ...


class _EventContainer(typing.Generic[System_Collections_ObjectModel__EventContainer_Callable, System_Collections_ObjectModel__EventContainer_ReturnType]):
    """This class is used to provide accurate autocomplete on events and cannot be imported."""

    def __call__(self, *args: typing.Any, **kwargs: typing.Any) -> System_Collections_ObjectModel__EventContainer_ReturnType:
        """Fires the event."""
        ...

    def __iadd__(self, item: System_Collections_ObjectModel__EventContainer_Callable) -> None:
        """Registers an event handler."""
        ...

    def __isub__(self, item: System_Collections_ObjectModel__EventContainer_Callable) -> None:
        """Unregisters an event handler."""
        ...





from typing import overload
import abc
import typing

import System
import System.Collections
import System.ComponentModel
import System.ComponentModel.Design
import System.ComponentModel.Design.Serialization
import System.IO
import System.Reflection

IServiceProvider = typing.Any
System_ComponentModel_Design_Serialization_MemberRelationship = typing.Any

System_ComponentModel_Design_Serialization__EventContainer_Callable = typing.TypeVar("System_ComponentModel_Design_Serialization__EventContainer_Callable")
System_ComponentModel_Design_Serialization__EventContainer_ReturnType = typing.TypeVar("System_ComponentModel_Design_Serialization__EventContainer_ReturnType")


class SerializationStore(System.Object, System.IDisposable, metaclass=abc.ABCMeta):
    """
    The SerializationStore class is an implementation-specific class that stores
    serialization data for the component serialization service. The
    service adds state to this serialization store. Once the store is
    closed it can be saved to a stream. A serialization store can be
    deserialized at a later date by the same type of serialization service.
    SerializationStore implements the IDisposable interface such that Dispose
    simply calls the Close method. Dispose is implemented as a private
    interface to avoid confusion. The IDisposable pattern is provided
    for languages that support a "using" syntax like C# and VB .NET.
    """

    @property
    @abc.abstractmethod
    def Errors(self) -> System.Collections.ICollection:
        """
        If there were errors generated during serialization or deserialization of the store, they will be
        added to this collection.
        """
        ...

    def Close(self) -> None:
        """
        The Close method closes this store and prevents any objects
        from being serialized into it. Once closed, the serialization store may be saved.
        """
        ...

    @overload
    def Dispose(self) -> None:
        """Disposes this object by calling the Close method."""
        ...

    @overload
    def Dispose(self, disposing: bool) -> None:
        """This method is protected."""
        ...

    def Save(self, stream: System.IO.Stream) -> None:
        """
        The Save method saves the store to the given stream. If the store
        is open, Save will automatically close it for you. You
        can call save as many times as you wish to save the store
        to different streams.
        """
        ...


class ComponentSerializationService(System.Object, metaclass=abc.ABCMeta):
    """
    This class serializes a set of components or serializable objects into a
    serialization store. The store can then be deserialized at a later
    date. ComponentSerializationService differs from other serialization
    schemes in that the serialization format is opaque, and it allows for
    partial serialization of objects. For example, you can choose to
    serialize only selected properties for an object.
    
    This class is abstract. Typically a DesignerLoader will provide a
    concrete implementation of this class and add it as a service to
    its DesignSurface. This allows objects to be serialized in the
    format best suited for them.
    """

    def CreateStore(self) -> System.ComponentModel.Design.Serialization.SerializationStore:
        """
        This method creates a new SerializationStore. The serialization store can
        be passed to any of the various Serialize methods to build up serialization
        state for a group of objects.
        """
        ...

    @overload
    def Deserialize(self, store: System.ComponentModel.Design.Serialization.SerializationStore) -> System.Collections.ICollection:
        """
        This method deserializes the given store to produce a collection of
        objects contained within it. If a container is provided, objects
        that are created that implement IComponent will be added to the container.
        """
        ...

    @overload
    def Deserialize(self, store: System.ComponentModel.Design.Serialization.SerializationStore, container: System.ComponentModel.IContainer) -> System.Collections.ICollection:
        """
        This method deserializes the given store to produce a collection of
        objects contained within it. If a container is provided, objects
        that are created that implement IComponent will be added to the container.
        """
        ...

    @overload
    def DeserializeTo(self, store: System.ComponentModel.Design.Serialization.SerializationStore, container: System.ComponentModel.IContainer, validateRecycledTypes: bool, applyDefaults: bool) -> None:
        """
        This method deserializes the given store, but rather than produce
        new objects, the data in the store is applied to an existing
        set of objects that are taken from the provided container. This
        allows the caller to pre-create an object however it sees fit. If
        an object has deserialization state and the object is not named in
        the set of existing objects, a new object will be created. If that
        object also implements IComponent, it will be added to the given
        container. Objects in the container must have names that
        match objects in the serialization store in order for an existing
        object to be used. If validateRecycledTypes is true it is guaranteed
        that the deserialization will only work if applied to an object of the
        same type.
        """
        ...

    @overload
    def DeserializeTo(self, store: System.ComponentModel.Design.Serialization.SerializationStore, container: System.ComponentModel.IContainer) -> None:
        ...

    @overload
    def DeserializeTo(self, store: System.ComponentModel.Design.Serialization.SerializationStore, container: System.ComponentModel.IContainer, validateRecycledTypes: bool) -> None:
        ...

    def LoadStore(self, stream: System.IO.Stream) -> System.ComponentModel.Design.Serialization.SerializationStore:
        """
        This method loads a SerializationStore and from the given
        stream. This store can then be used to deserialize objects by passing it to
        the various Deserialize methods.
        """
        ...

    def Serialize(self, store: System.ComponentModel.Design.Serialization.SerializationStore, value: typing.Any) -> None:
        """
        This method serializes the given object to the store. The store
        can be used to serialize more than one object by calling this method
        more than once.
        """
        ...

    def SerializeAbsolute(self, store: System.ComponentModel.Design.Serialization.SerializationStore, value: typing.Any) -> None:
        """
        Normal serialization only serializes values that differ from the component's default state.
        This provides the most compact serialization mechanism but assumes that during deserialization
        a new, freshly created object will be used. If an existing object is used properties that
        contained default values during serialization would not be reset back to their defaults.
        The SerializeAbsolute method does not require this assumption on the deserializing end.
        Instead, it saves data in the serialization store about default values as well so that
        deserialization can reset non-default properties back to their default values. This is
        especially true for collection objects, where the collections are either cleared and
        items re-added, or individual items are removed and added.
        """
        ...

    def SerializeMember(self, store: System.ComponentModel.Design.Serialization.SerializationStore, owningObject: typing.Any, member: System.ComponentModel.MemberDescriptor) -> None:
        """
        This method serializes the given member on the given object. This method
        can be invoked multiple times for the same object to build up a list of
        serialized members within the serialization store. The member generally
        has to be a property or an event.
        """
        ...

    def SerializeMemberAbsolute(self, store: System.ComponentModel.Design.Serialization.SerializationStore, owningObject: typing.Any, member: System.ComponentModel.MemberDescriptor) -> None:
        """
        This method serializes the given member on the given object,
        but also serializes the member if it contains the default value.
        Note that for some members, containing the default value and setting
        the same value back to the member are different concepts. For example,
        if a property inherits its value from a parent object if no local value
        is set, setting the value back to the property can may not be what is desired.
        SerializeMemberAbsolute takes this into account and would clear the state of
        the property in this case.
        """
        ...


class IDesignerLoaderService(metaclass=abc.ABCMeta):
    """
    This interface may be optionally implemented by the designer loader to provide
    load services to outside components. It provides support for asynchronous loading
    of the designer and allows other objects to initiate a reload of othe
    design surface. Designer loaders do not need to implement this but it is
    recommended. We do not directly put this on DesignerLoader so we can prevent
    outside objects from interacting with the main methods of a designer loader.
    These should only be called by the designer host.
    """

    def AddLoadDependency(self) -> None:
        """
        Adds a load dependency to this loader. This indicates that some other
        object is also participating in the load, and that the designer loader
        should not call EndLoad on the loader host until all load dependencies
        have called DependentLoadComplete on the designer loader.
        """
        ...

    def DependentLoadComplete(self, successful: bool, errorCollection: System.Collections.ICollection) -> None:
        """
        This is called by any object that has previously called
        AddLoadDependency to signal that the dependent load has completed.
        The caller should pass either an empty collection or null to indicate
        a successful load, or a collection of exceptions that indicate the
        reason(s) for failure.
        """
        ...

    def Reload(self) -> bool:
        """
        This can be called by an outside object to request that the loader
        reload the design document. If it supports reloading and wants to
        comply with the reload, the designer loader should return true. Otherwise
        it should return false, indicating that the reload will not occur.
        Callers should not rely on the reload happening immediately; the
        designer loader may schedule this for some other time, or it may
        try to reload at once.
        """
        ...


class RootDesignerSerializerAttribute(System.Attribute):
    """
    This attribute can be placed on a class to indicate what serialization
    object should be used to serialize the class at design time if it is
    being used as a root object.
    
    RootDesignerSerializerAttribute has been deprecated. Use DesignerSerializerAttribute instead. For example, to specify a root designer for CodeDom, use DesignerSerializerAttribute(...,typeof(TypeCodeDomSerializer)) instead.
    """

    @property
    def Reloadable(self) -> bool:
        """
        Indicates that this root serializer supports reloading. If false, the design document
        will not automatically perform a reload on behalf of the user. It will be the user's
        responsibility to reload the document themselves.
        """
        ...

    @property
    def SerializerTypeName(self) -> str:
        """Retrieves the fully qualified type name of the serializer."""
        ...

    @property
    def SerializerBaseTypeName(self) -> str:
        """Retrieves the fully qualified type name of the serializer base type."""
        ...

    @property
    def TypeId(self) -> System.Object:
        """
        This defines a unique ID for this attribute type. It is used
        by filtering algorithms to identify two attributes that are
        the same type. For most attributes, this just returns the
        Type instance for the attribute. EditorAttribute overrides
        this to include the type of the editor base type.
        """
        ...

    @overload
    def __init__(self, serializerType: typing.Type, baseSerializerType: typing.Type, reloadable: bool) -> None:
        """Creates a new designer serialization attribute."""
        ...

    @overload
    def __init__(self, serializerTypeName: str, baseSerializerType: typing.Type, reloadable: bool) -> None:
        """Creates a new designer serialization attribute."""
        ...

    @overload
    def __init__(self, serializerTypeName: str, baseSerializerTypeName: str, reloadable: bool) -> None:
        """Creates a new designer serialization attribute."""
        ...


class IDesignerLoaderHost(System.ComponentModel.Design.IDesignerHost, metaclass=abc.ABCMeta):
    """
    IDesignerLoaderHost. This is an extension of IDesignerHost that is passed
    to the designer loader in the BeginLoad method. It is isolated from
    IDesignerHost to emphasize that all loading and reloading of the design
    document actually should be initiated by the designer loader, and not by
    the designer host. However, the loader must inform the designer host that
    it wishes to invoke a load or reload.
    """

    def EndLoad(self, baseClassName: str, successful: bool, errorCollection: System.Collections.ICollection) -> None:
        """
        This is called by the designer loader to indicate that the load has
        terminated. If there were errors, they should be passed in the errorCollection
        as a collection of exceptions (if they are not exceptions the designer
        loader host may just call ToString on them). If the load was successful then
        errorCollection should either be null or contain an empty collection.
        """
        ...

    def Reload(self) -> None:
        """
        This is called by the designer loader when it wishes to reload the
        design document. The reload will happen immediately so the caller
        should ensure that it is in a state where BeginLoad may be called again.
        """
        ...


class IDesignerLoaderHost2(System.ComponentModel.Design.Serialization.IDesignerLoaderHost, metaclass=abc.ABCMeta):
    """
    IgnoreErrorsDuringReload - specifies whether errors should be ignored when Reload() is called.
                      We only allow to set to true if we CanReloadWithErrors. If we cannot
                      we simply ignore rather than throwing an exception. We probably should,
                      but we are avoiding localization.
    CanReloadWithErrors - specifies whether it is possible to reload with errors. There are certain
                 scenarios where errors cannot be ignored.
    """

    @property
    @abc.abstractmethod
    def IgnoreErrorsDuringReload(self) -> bool:
        ...

    @IgnoreErrorsDuringReload.setter
    @abc.abstractmethod
    def IgnoreErrorsDuringReload(self, value: bool):
        ...

    @property
    @abc.abstractmethod
    def CanReloadWithErrors(self) -> bool:
        ...

    @CanReloadWithErrors.setter
    @abc.abstractmethod
    def CanReloadWithErrors(self, value: bool):
        ...


class DefaultSerializationProviderAttribute(System.Attribute):
    """
    The default serialization provider attribute is placed on a serializer
    to indicate the class to use as a default provider of that type of
    serializer. To be a default serialization provider, a class must
    implement IDesignerSerilaizationProvider and have an empty
    constructor. The class itself can be internal to the assembly.
    """

    @property
    def ProviderTypeName(self) -> str:
        """Returns the type name for the default serialization provider."""
        ...

    @overload
    def __init__(self, providerType: typing.Type) -> None:
        """Creates a new DefaultSerializationProviderAttribute"""
        ...

    @overload
    def __init__(self, providerTypeName: str) -> None:
        """Creates a new DefaultSerializationProviderAttribute"""
        ...


class ContextStack(System.Object):
    """
    A context stack is an object that can be used by serializers
    to push various context objects. Serialization is often
    a deeply nested operation, involving many different
    serialization classes. These classes often need additional
    context information when performing serialization. As
    an example, an object with a property named "Enabled" may have
    a data type of System.Boolean. If a serializer is writing
    this value to a data stream it may want to know what property
    it is writing. It won't have this information, however, because
    it is only instructed to write the boolean value. In this
    case the parent serializer may push a PropertyDescriptor
    pointing to the "Enabled" property on the context stack.
    What objects get pushed on this stack are up to the
    individual serializer objects.
    """

    @property
    def Current(self) -> System.Object:
        """
        Retrieves the current object on the stack, or null
        if no objects have been pushed.
        """
        ...

    @overload
    def __getitem__(self, level: int) -> typing.Any:
        """
        Retrieves the object on the stack at the given
        level, or null if no object exists at that level.
        """
        ...

    @overload
    def __getitem__(self, type: typing.Type) -> typing.Any:
        """
        Retrieves the first object on the stack that
        inherits from or implements the given type, or
        null if no object on the stack implements the type.
        """
        ...

    def Append(self, context: typing.Any) -> None:
        """
        Appends an object to the end of the stack, rather than pushing it
        onto the top of the stack. This method allows a serializer to communicate
        with other serializers by adding contextual data that does not have to
        be popped in order. There is no way to remove an object that was
        appended to the end of the stack without popping all other objects.
        """
        ...

    def Pop(self) -> System.Object:
        """
        Pops the current object off of the stack, returning
        its value.
        """
        ...

    def Push(self, context: typing.Any) -> None:
        """Pushes the given object onto the stack."""
        ...


class IDesignerSerializationProvider(metaclass=abc.ABCMeta):
    """
    This interface defines a custom serialization provider. This
    allows outside objects to control serialization by providing
    their own serializer objects.
    """

    def GetSerializer(self, manager: System.ComponentModel.Design.Serialization.IDesignerSerializationManager, currentSerializer: typing.Any, objectType: typing.Type, serializerType: typing.Type) -> System.Object:
        """
        This will be called by the serialization manager when it
        is trying to locate a serializer for an object type.
        If this serialization provider can provide a serializer
        that is of the correct type, it should return it.
        Otherwise, it should return null.
        
        In order to break order dependencies between multiple
        serialization providers the serialization manager will
        loop through all serialization providers until the
        serializer returned reaches steady state. Because
        of this you should always check currentSerializer
        before returning a new serializer. If currentSerializer
        is an instance of your serializer, then you should
        either return it or return null to prevent an infinite
        loop.
        """
        ...


class ResolveNameEventArgs(System.EventArgs):
    """
    EventArgs for the ResolveNameEventHandler. This event is used
    by the serialization process to match a name to an object
    instance.
    """

    @property
    def Name(self) -> str:
        """The name of the object that needs to be resolved."""
        ...

    @property
    def Value(self) -> System.Object:
        """The object that matches the name."""
        ...

    @Value.setter
    def Value(self, value: System.Object):
        """The object that matches the name."""
        ...

    def __init__(self, name: str) -> None:
        """Creates a new resolve name event args object."""
        ...


class IDesignerSerializationManager(IServiceProvider, metaclass=abc.ABCMeta):
    """
    This interface is passed to a designer serializer to provide
    assistance in the serialization process.
    """

    @property
    @abc.abstractmethod
    def Context(self) -> System.ComponentModel.Design.Serialization.ContextStack:
        """
        The Context property provides a user-defined storage area
        implemented as a stack. This storage area is a useful way
        to provide communication across serializers, as serialization
        is a generally hierarchical process.
        """
        ...

    @property
    @abc.abstractmethod
    def Properties(self) -> System.ComponentModel.PropertyDescriptorCollection:
        """
        The Properties property provides a set of custom properties
        the serialization manager may surface. The set of properties
        exposed here is defined by the implementor of
        IDesignerSerializationManager.
        """
        ...

    @property
    @abc.abstractmethod
    def ResolveName(self) -> _EventContainer[typing.Callable[[System.Object, System.ComponentModel.Design.Serialization.ResolveNameEventArgs], None], None]:
        """
        ResolveName event. This event
        is raised when GetName is called, but the name is not found
        in the serialization manager's name table. It provides a
        way for a serializer to demand-create an object so the serializer
        does not have to order object creation by dependency. This
        delegate is cleared immediately after serialization or deserialization
        is complete.
        """
        ...

    @ResolveName.setter
    @abc.abstractmethod
    def ResolveName(self, value: _EventContainer[typing.Callable[[System.Object, System.ComponentModel.Design.Serialization.ResolveNameEventArgs], None], None]):
        """
        ResolveName event. This event
        is raised when GetName is called, but the name is not found
        in the serialization manager's name table. It provides a
        way for a serializer to demand-create an object so the serializer
        does not have to order object creation by dependency. This
        delegate is cleared immediately after serialization or deserialization
        is complete.
        """
        ...

    @property
    @abc.abstractmethod
    def SerializationComplete(self) -> _EventContainer[typing.Callable[[System.Object, System.EventArgs], None], None]:
        """
        This event is raised when serialization or deserialization
        has been completed. Generally, serialization code should
        be written to be stateless. Should some sort of state
        be necessary to maintain, a serializer can listen to
        this event to know when that state should be cleared.
        An example of this is if a serializer needs to write
        to another file, such as a resource file. In this case
        it would be inefficient to design the serializer
        to close the file when finished because serialization of
        an object graph generally requires several serializers.
        The resource file would be opened and closed many times.
        Instead, the resource file could be accessed through
        an object that listened to the SerializationComplete
        event, and that object could close the resource file
        at the end of serialization.
        """
        ...

    @SerializationComplete.setter
    @abc.abstractmethod
    def SerializationComplete(self, value: _EventContainer[typing.Callable[[System.Object, System.EventArgs], None], None]):
        """
        This event is raised when serialization or deserialization
        has been completed. Generally, serialization code should
        be written to be stateless. Should some sort of state
        be necessary to maintain, a serializer can listen to
        this event to know when that state should be cleared.
        An example of this is if a serializer needs to write
        to another file, such as a resource file. In this case
        it would be inefficient to design the serializer
        to close the file when finished because serialization of
        an object graph generally requires several serializers.
        The resource file would be opened and closed many times.
        Instead, the resource file could be accessed through
        an object that listened to the SerializationComplete
        event, and that object could close the resource file
        at the end of serialization.
        """
        ...

    def AddSerializationProvider(self, provider: System.ComponentModel.Design.Serialization.IDesignerSerializationProvider) -> None:
        """
        This method adds a custom serialization provider to the
        serialization manager. A custom serialization provider will
        get the opportunity to return a serializer for a data type
        before the serialization manager looks in the type's
        metadata.
        """
        ...

    def CreateInstance(self, type: typing.Type, arguments: System.Collections.ICollection, name: str, addToContainer: bool) -> System.Object:
        """
        Creates an instance of the given type and adds it to a collection
        of named instances. Objects that implement IComponent will be
        added to the design time container if addToContainer is true.
        """
        ...

    def GetInstance(self, name: str) -> System.Object:
        """
        Retrieves an instance of a created object of the given name, or
        null if that object does not exist.
        """
        ...

    def GetName(self, value: typing.Any) -> str:
        """
        Retrieves a name for the specified object, or null if the object
        has no name.
        """
        ...

    def GetSerializer(self, objectType: typing.Type, serializerType: typing.Type) -> System.Object:
        """
        Retrieves a serializer of the requested type for the given
        object type.
        """
        ...

    def GetType(self, typeName: str) -> typing.Type:
        """Retrieves a type of the given name."""
        ...

    def RemoveSerializationProvider(self, provider: System.ComponentModel.Design.Serialization.IDesignerSerializationProvider) -> None:
        """Removes a previously added serialization provider."""
        ...

    def ReportError(self, errorInformation: typing.Any) -> None:
        """
        Reports a non-fatal error in serialization. The serialization
        manager may implement a logging scheme to alert the caller
        to all non-fatal errors at once. If it doesn't, it should
        immediately throw in this method, which should abort
        serialization.
        Serialization may continue after calling this function.
        """
        ...

    def SetName(self, instance: typing.Any, name: str) -> None:
        """
        Provides a way to set the name of an existing object.
        This is useful when it is necessary to create an
        instance of an object without going through CreateInstance.
        An exception will be thrown if you try to rename an existing
        object or if you try to give a new object a name that
        is already taken.
        """
        ...


class MemberRelationship(System.IEquatable[System_ComponentModel_Design_Serialization_MemberRelationship]):
    """This class represents a single relationship between an object and a member."""

    Empty: System.ComponentModel.Design.Serialization.MemberRelationship

    @property
    def IsEmpty(self) -> bool:
        """Returns true if this relationship is empty."""
        ...

    @property
    def Member(self) -> System.ComponentModel.MemberDescriptor:
        """The member in this relationship."""
        ...

    @property
    def Owner(self) -> System.Object:
        """The object owning the member."""
        ...

    def __init__(self, owner: typing.Any, member: System.ComponentModel.MemberDescriptor) -> None:
        """Creates a new member relationship."""
        ...

    @overload
    def Equals(self, obj: typing.Any) -> bool:
        """Infrastructure support to make this a first class struct"""
        ...

    @overload
    def Equals(self, other: System.ComponentModel.Design.Serialization.MemberRelationship) -> bool:
        """Infrastructure support to make this a first class struct"""
        ...

    def GetHashCode(self) -> int:
        """Infrastructure support to make this a first class struct"""
        ...


class MemberRelationshipService(System.Object, metaclass=abc.ABCMeta):
    """
    A member relationship service is used by a serializer to announce that one
    property is related to a property on another object. Consider a code
    based serialization scheme where code is of the following form:
    
    object1.Property1 = object2.Property2
    
    Upon interpretation of this code, Property1 on object1 will be
    set to the return value of object2.Property2. But the relationship
    between these two objects is lost. Serialization schemes that
    wish to maintain this relationship may install a MemberRelationshipService
    into the serialization manager. When an object is deserialized
    this service will be notified of these relationships. It is up to the service
    to act on these notifications if it wishes. During serialization, the
    service is also consulted. If a relationship exists the same
    relationship is maintained by the serializer.
    """

    @overload
    def __getitem__(self, source: System.ComponentModel.Design.Serialization.MemberRelationship) -> System.ComponentModel.Design.Serialization.MemberRelationship:
        """
        Returns the current relationship associated with the source, or MemberRelationship.Empty if
        there is no relationship. Also sets a relationship between two objects. Empty
        can also be passed as the property value, in which case the relationship will
        be cleared.
        """
        ...

    @overload
    def __getitem__(self, sourceOwner: typing.Any, sourceMember: System.ComponentModel.MemberDescriptor) -> System.ComponentModel.Design.Serialization.MemberRelationship:
        """
        Returns the current relationship associated with the source, or null if there is no relationship.
        Also sets a relationship between two objects. Null can be passed as the property value, in which
        case the relationship will be cleared.
        """
        ...

    @overload
    def __setitem__(self, source: System.ComponentModel.Design.Serialization.MemberRelationship, value: System.ComponentModel.Design.Serialization.MemberRelationship) -> None:
        """
        Returns the current relationship associated with the source, or MemberRelationship.Empty if
        there is no relationship. Also sets a relationship between two objects. Empty
        can also be passed as the property value, in which case the relationship will
        be cleared.
        """
        ...

    @overload
    def __setitem__(self, sourceOwner: typing.Any, sourceMember: System.ComponentModel.MemberDescriptor, value: System.ComponentModel.Design.Serialization.MemberRelationship) -> None:
        """
        Returns the current relationship associated with the source, or null if there is no relationship.
        Also sets a relationship between two objects. Null can be passed as the property value, in which
        case the relationship will be cleared.
        """
        ...

    def GetRelationship(self, source: System.ComponentModel.Design.Serialization.MemberRelationship) -> System.ComponentModel.Design.Serialization.MemberRelationship:
        """
        This is the implementation API for returning relationships. The default implementation stores the
        relationship in a table. Relationships are stored weakly, so they do not keep an object alive.
        
        This method is protected.
        """
        ...

    def SetRelationship(self, source: System.ComponentModel.Design.Serialization.MemberRelationship, relationship: System.ComponentModel.Design.Serialization.MemberRelationship) -> None:
        """
        This is the implementation API for returning relationships. The default implementation stores the
        relationship in a table. Relationships are stored weakly, so they do not keep an object alive. Empty can be
        passed in for relationship to remove the relationship.
        
        This method is protected.
        """
        ...

    def SupportsRelationship(self, source: System.ComponentModel.Design.Serialization.MemberRelationship, relationship: System.ComponentModel.Design.Serialization.MemberRelationship) -> bool:
        """Returns true if the provided relationship is supported."""
        ...


class IDesignerSerializationService(metaclass=abc.ABCMeta):
    """
    This service provides a way to exchange a collection of objects
    for a serializable object that represents them. The returned
    object contains live references to objects in the collection.
    This returned object can then be passed to any runtime
    serialization mechanism. The object itself serializes
    components the same way designers write source for them; by picking
    them apart property by property. Many objects do not support
    runtime serialization because their internal state cannot be
    adequately duplicated. All components that support a designer,
    however, must support serialization by walking their public
    properties, methods and events. This interface uses this
    technique to convert a collection of components into a single
    opaque object that does support runtime serialization.
    """

    def Deserialize(self, serializationData: typing.Any) -> System.Collections.ICollection:
        """
        Deserializes the provided serialization data object and
        returns a collection of objects contained within that data.
        """
        ...

    def Serialize(self, objects: System.Collections.ICollection) -> System.Object:
        """
        Serializes the given collection of objects and
        stores them in an opaque serialization data object.
        The returning object fully supports runtime serialization.
        """
        ...


class InstanceDescriptor(System.Object):
    """
    EventArgs for the ResolveNameEventHandler. This event is used
    by the serialization process to match a name to an object
    instance.
    """

    @property
    def Arguments(self) -> System.Collections.ICollection:
        """
        The collection of arguments that should be passed to
        MemberInfo in order to create an instance.
        """
        ...

    @property
    def IsComplete(self) -> bool:
        """
        Determines if the contents of this instance descriptor completely identify the instance.
        This will normally be the case, but some objects may be too complex for a single method
        or constructor to represent. IsComplete can be used to identify these objects and take
        additional steps to further describe their state.
        """
        ...

    @property
    def MemberInfo(self) -> System.Reflection.MemberInfo:
        """
        The MemberInfo object that was passed into the constructor
        of this InstanceDescriptor.
        """
        ...

    @overload
    def __init__(self, member: System.Reflection.MemberInfo, arguments: System.Collections.ICollection) -> None:
        """Creates a new InstanceDescriptor."""
        ...

    @overload
    def __init__(self, member: System.Reflection.MemberInfo, arguments: System.Collections.ICollection, isComplete: bool) -> None:
        """Creates a new InstanceDescriptor."""
        ...

    def Invoke(self) -> System.Object:
        """
        Invokes this instance descriptor, returning the object
        the descriptor describes.
        """
        ...


class DesignerLoader(System.Object, metaclass=abc.ABCMeta):
    """
    DesignerLoader. This class is responsible for loading a designer document.
    Where and how this load occurs is a private matter for the designer loader.
    The designer loader will be handed to an IDesignerHost instance. This instance,
    when it is ready to load the document, will call BeginLoad, passing an instance
    of IDesignerLoaderHost. The designer loader will load up the design surface
    using the host interface, and call EndLoad on the interface when it is done.
    The error collection passed into EndLoad should be empty or null to indicate a
    successful load, or it should contain a collection of exceptions that
    describe the error.
    
    Once a document is loaded, the designer loader is also responsible for
    writing any changes made to the document back whatever storage the
    loader used when loading the document.
    """

    @property
    def Loading(self) -> bool:
        """
        Returns true when the designer is in the process of loading. Clients that are
        sinking notifications from the designer often want to ignore them while the designer is loading
        and only respond to them if they result from user interactions.
        """
        ...

    def BeginLoad(self, host: System.ComponentModel.Design.Serialization.IDesignerLoaderHost) -> None:
        """
        Called by the designer host to begin the loading process. The designer
        host passes in an instance of a designer loader host (which is typically
        the same object as the designer host. This loader host allows
        the designer loader to reload the design document and also allows
        the designer loader to indicate that it has finished loading the
        design document.
        """
        ...

    def Dispose(self) -> None:
        """
        Disposes this designer loader. The designer host will call this method
        when the design document itself is being destroyed. Once called, the
        designer loader will never be called again.
        """
        ...

    def Flush(self) -> None:
        """
        The designer host will call this periodically when it wants to
        ensure that any changes that have been made to the document
        have been saved by the designer loader. This method allows
        designer loaders to implement a lazy-write scheme to improve
        performance. The default implementation does nothing.
        """
        ...


class INameCreationService(metaclass=abc.ABCMeta):
    """
    This service may be provided by a designer loader to provide
    a way for the designer to fabricate new names for objects.
    If this service isn't available the designer will choose a
    default implementation.
    """

    def CreateName(self, container: System.ComponentModel.IContainer, dataType: typing.Type) -> str:
        """
        Creates a new name that is unique to all the components
        in the given container. The name will be used to create
        an object of the given data type, so the service may
        derive a name from the data type's name. The container
        parameter can be null if no container search is needed.
        """
        ...

    def IsValidName(self, name: str) -> bool:
        """
        Determines if the given name is valid. A name
        creation service may have rules defining a valid
        name, and this method allows the service to enforce
        those rules.
        """
        ...

    def ValidateName(self, name: str) -> None:
        """
        Determines if the given name is valid. A name
        creation service may have rules defining a valid
        name, and this method allows the service to enforce
        those rules. It is similar to IsValidName, except
        that this method will throw an exception if the
        name is invalid. This allows implementors to provide
        rich information in the exception message.
        """
        ...


class _EventContainer(typing.Generic[System_ComponentModel_Design_Serialization__EventContainer_Callable, System_ComponentModel_Design_Serialization__EventContainer_ReturnType]):
    """This class is used to provide accurate autocomplete on events and cannot be imported."""

    def __call__(self, *args: typing.Any, **kwargs: typing.Any) -> System_ComponentModel_Design_Serialization__EventContainer_ReturnType:
        """Fires the event."""
        ...

    def __iadd__(self, item: System_ComponentModel_Design_Serialization__EventContainer_Callable) -> None:
        """Registers an event handler."""
        ...

    def __isub__(self, item: System_ComponentModel_Design_Serialization__EventContainer_Callable) -> None:
        """Unregisters an event handler."""
        ...





from typing import overload
import abc
import datetime
import typing

import QuantConnect
import QuantConnect.Brokerages
import QuantConnect.Brokerages.Oanda
import QuantConnect.Data
import QuantConnect.Data.Market
import QuantConnect.Interfaces
import QuantConnect.Orders
import QuantConnect.Packets
import QuantConnect.Securities
import System
import System.Collections.Concurrent
import System.Collections.Generic


class Environment(System.Enum):
    """Represents different environments available for the REST API."""

    Practice = 0
    """A stable environment; recommended for testing with your fxTrade Practice account and your personal access token."""

    Trade = 1
    """A stable environment; recommended for production-ready code to execute with your fxTrade account and your personal access token."""


class OandaBrokerage(QuantConnect.Brokerages.Brokerage, QuantConnect.Interfaces.IDataQueueHandler):
    """Oanda Brokerage implementation"""

    MaxBarsPerRequest: int = 5000
    """The maximum number of bars per historical data request"""

    @property
    def IsConnected(self) -> bool:
        ...

    @property
    def AccountBaseCurrency(self) -> str:
        """Returns the brokerage account's base currency"""
        ...

    @AccountBaseCurrency.setter
    def AccountBaseCurrency(self, value: str):
        """Returns the brokerage account's base currency"""
        ...

    @overload
    def __init__(self) -> None:
        """Initializes a new instance of the OandaBrokerage class."""
        ...

    @overload
    def __init__(self, orderProvider: QuantConnect.Securities.IOrderProvider, securityProvider: QuantConnect.Securities.ISecurityProvider, aggregator: QuantConnect.Data.IDataAggregator, environment: QuantConnect.Brokerages.Oanda.Environment, accessToken: str, accountId: str, agent: str = ...) -> None:
        """
        Initializes a new instance of the OandaBrokerage class.
        
        :param orderProvider: The order provider.
        :param securityProvider: The holdings provider.
        :param aggregator: consolidate ticks
        :param environment: The Oanda environment (Trade or Practice)
        :param accessToken: The Oanda access token (can be the user's personal access token or the access token obtained with OAuth by QC on behalf of the user)
        :param accountId: The account identifier.
        :param agent: The Oanda agent string
        """
        ...

    def CancelOrder(self, order: QuantConnect.Orders.Order) -> bool:
        """
        Cancels the order with the specified ID
        
        :param order: The order to cancel
        :returns: True if the request was made for the order to be canceled, false otherwise.
        """
        ...

    def Connect(self) -> None:
        """Connects the client to the broker's remote servers"""
        ...

    def Disconnect(self) -> None:
        """Disconnects the client from the broker's remote servers"""
        ...

    def DownloadQuoteBars(self, symbol: typing.Union[QuantConnect.Symbol, str], startTimeUtc: typing.Union[datetime.datetime, datetime.date], endTimeUtc: typing.Union[datetime.datetime, datetime.date], resolution: QuantConnect.Resolution, requestedTimeZone: typing.Any) -> System.Collections.Generic.IEnumerable[QuantConnect.Data.Market.QuoteBar]:
        """
        Downloads a list of QuoteBars at the requested resolution
        
        :param symbol: The symbol
        :param startTimeUtc: The starting time (UTC)
        :param endTimeUtc: The ending time (UTC)
        :param resolution: The requested resolution
        :param requestedTimeZone: The requested timezone for the data
        :returns: The list of bars.
        """
        ...

    def DownloadTradeBars(self, symbol: typing.Union[QuantConnect.Symbol, str], startTimeUtc: typing.Union[datetime.datetime, datetime.date], endTimeUtc: typing.Union[datetime.datetime, datetime.date], resolution: QuantConnect.Resolution, requestedTimeZone: typing.Any) -> System.Collections.Generic.IEnumerable[QuantConnect.Data.Market.TradeBar]:
        """
        Downloads a list of TradeBars at the requested resolution
        
        :param symbol: The symbol
        :param startTimeUtc: The starting time (UTC)
        :param endTimeUtc: The ending time (UTC)
        :param resolution: The requested resolution
        :param requestedTimeZone: The requested timezone for the data
        :returns: The list of bars.
        """
        ...

    def GetAccountHoldings(self) -> System.Collections.Generic.List[QuantConnect.Holding]:
        """
        Gets all holdings for the account
        
        :returns: The current holdings from the account.
        """
        ...

    def GetCashBalance(self) -> System.Collections.Generic.List[QuantConnect.Securities.CashAmount]:
        """
        Gets the current cash balance for each currency held in the brokerage account
        
        :returns: The current cash balance for each currency available for trading.
        """
        ...

    @staticmethod
    def GetDateTimeFromString(time: str) -> datetime.datetime:
        ...

    def GetHistory(self, request: QuantConnect.Data.HistoryRequest) -> System.Collections.Generic.IEnumerable[QuantConnect.Data.BaseData]:
        """
        Gets the history for the requested security
        
        :param request: The historical data request
        :returns: An enumerable of bars covering the span specified in the request.
        """
        ...

    def GetOpenOrders(self) -> System.Collections.Generic.List[QuantConnect.Orders.Order]:
        """
        Gets all open orders on the account.
        NOTE: The order objects returned do not have QC order IDs.
        
        :returns: The open orders returned from Oanda.
        """
        ...

    def GetRates(self, instrument: str) -> QuantConnect.Data.Market.Tick:
        """
        Retrieves the current quotes for an instrument
        
        :param instrument: the instrument to check
        :returns: Returns a Tick object with the current bid/ask prices for the instrument.
        """
        ...

    def PlaceOrder(self, order: QuantConnect.Orders.Order) -> bool:
        """
        Places a new order and assigns a new broker ID to the order
        
        :param order: The order to be placed
        :returns: True if the request for a new order has been placed, false otherwise.
        """
        ...

    def SetJob(self, job: QuantConnect.Packets.LiveNodePacket) -> None:
        ...

    def Subscribe(self, dataConfig: QuantConnect.Data.SubscriptionDataConfig, newDataAvailableHandler: typing.Callable[[System.Object, System.EventArgs], None]) -> System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]:
        """
        Subscribe to the specified configuration
        
        :param dataConfig: defines the parameters to subscribe to a data feed
        :param newDataAvailableHandler: handler to be fired on new data available
        :returns: The new enumerator for this subscription request.
        """
        ...

    def Unsubscribe(self, dataConfig: QuantConnect.Data.SubscriptionDataConfig) -> None:
        """
        Removes the specified configuration
        
        :param dataConfig: Subscription config to be removed
        """
        ...

    def UpdateOrder(self, order: QuantConnect.Orders.Order) -> bool:
        """
        Updates the order with the same id
        
        :param order: The new order information
        :returns: True if the request was made for the order to be updated, false otherwise.
        """
        ...


class OandaSymbolMapper(System.Object, QuantConnect.Brokerages.ISymbolMapper):
    """Provides the mapping between Lean symbols and Oanda symbols."""

    KnownTickers: System.Collections.Generic.HashSet[str] = ...
    """The list of known Oanda symbols."""

    def GetBrokerageSecurityType(self, brokerageSymbol: str) -> int:
        """
        Returns the security type for an Oanda symbol
        
        :param brokerageSymbol: The Oanda symbol
        :returns: The security type. This method returns the int value of a member of the QuantConnect.SecurityType enum.
        """
        ...

    def GetBrokerageSymbol(self, symbol: typing.Union[QuantConnect.Symbol, str]) -> str:
        """
        Converts a Lean symbol instance to an Oanda symbol
        
        :param symbol: A Lean symbol instance
        :returns: The Oanda symbol.
        """
        ...

    def GetLeanSecurityType(self, leanSymbol: str) -> int:
        """
        Returns the security type for a Lean symbol
        
        :param leanSymbol: The Lean symbol
        :returns: The security type. This method returns the int value of a member of the QuantConnect.SecurityType enum.
        """
        ...

    def GetLeanSymbol(self, brokerageSymbol: str, securityType: QuantConnect.SecurityType, market: str, expirationDate: typing.Union[datetime.datetime, datetime.date] = ..., strike: float = 0, optionRight: QuantConnect.OptionRight = 0) -> QuantConnect.Symbol:
        """
        Converts an Oanda symbol to a Lean symbol instance
        
        :param brokerageSymbol: The Oanda symbol
        :param securityType: The security type
        :param market: The market
        :param expirationDate: Expiration date of the security(if applicable)
        :param strike: The strike of the security (if applicable)
        :param optionRight: The option right of the security (if applicable)
        :returns: A new Lean Symbol instance.
        """
        ...

    def IsKnownBrokerageSymbol(self, brokerageSymbol: str) -> bool:
        """
        Checks if the symbol is supported by Oanda
        
        :param brokerageSymbol: The Oanda symbol
        :returns: True if Oanda supports the symbol.
        """
        ...

    def IsKnownLeanSymbol(self, symbol: typing.Union[QuantConnect.Symbol, str]) -> bool:
        """
        Checks if the symbol is supported by Oanda
        
        :param symbol: The Lean symbol
        :returns: True if Oanda supports the symbol.
        """
        ...


class OandaRestApiBase(QuantConnect.Brokerages.Brokerage, QuantConnect.Interfaces.IDataQueueHandler, metaclass=abc.ABCMeta):
    """Oanda REST API base class"""

    @property
    def Locker(self) -> System.Object:
        """
        This lock is used to sync 'PlaceOrder' and callback 'OnTransactionDataReceived'
        
        This field is protected.
        """
        ...

    @property
    def PendingFilledMarketOrders(self) -> System.Collections.Concurrent.ConcurrentDictionary[int, QuantConnect.Orders.OrderStatus]:
        """
        This container is used to keep pending to be filled market orders, so when the callback comes in we send the filled event
        
        This field is protected.
        """
        ...

    @property
    def PricingConnectionHandler(self) -> QuantConnect.Brokerages.IConnectionHandler:
        """
        The connection handler for pricing
        
        This field is protected.
        """
        ...

    @property
    def TransactionsConnectionHandler(self) -> QuantConnect.Brokerages.IConnectionHandler:
        """
        The connection handler for transactions
        
        This field is protected.
        """
        ...

    @property
    def SubscribedSymbols(self) -> System.Collections.Generic.IEnumerable[QuantConnect.Symbol]:
        """
        The list of currently subscribed symbols
        
        This property is protected.
        """
        ...

    @property
    def SymbolMapper(self) -> QuantConnect.Brokerages.Oanda.OandaSymbolMapper:
        """
        The symbol mapper
        
        This field is protected.
        """
        ...

    @SymbolMapper.setter
    def SymbolMapper(self, value: QuantConnect.Brokerages.Oanda.OandaSymbolMapper):
        """
        The symbol mapper
        
        This field is protected.
        """
        ...

    @property
    def OrderProvider(self) -> QuantConnect.Securities.IOrderProvider:
        """
        The order provider
        
        This field is protected.
        """
        ...

    @OrderProvider.setter
    def OrderProvider(self, value: QuantConnect.Securities.IOrderProvider):
        """
        The order provider
        
        This field is protected.
        """
        ...

    @property
    def SecurityProvider(self) -> QuantConnect.Securities.ISecurityProvider:
        """
        The security provider
        
        This field is protected.
        """
        ...

    @SecurityProvider.setter
    def SecurityProvider(self, value: QuantConnect.Securities.ISecurityProvider):
        """
        The security provider
        
        This field is protected.
        """
        ...

    @property
    def Aggregator(self) -> QuantConnect.Data.IDataAggregator:
        """
        The data aggregator
        
        This field is protected.
        """
        ...

    @Aggregator.setter
    def Aggregator(self, value: QuantConnect.Data.IDataAggregator):
        """
        The data aggregator
        
        This field is protected.
        """
        ...

    @property
    def Environment(self) -> QuantConnect.Brokerages.Oanda.Environment:
        """
        The Oanda enviroment
        
        This field is protected.
        """
        ...

    @Environment.setter
    def Environment(self, value: QuantConnect.Brokerages.Oanda.Environment):
        """
        The Oanda enviroment
        
        This field is protected.
        """
        ...

    @property
    def AccessToken(self) -> str:
        """
        The Oanda access token
        
        This field is protected.
        """
        ...

    @AccessToken.setter
    def AccessToken(self, value: str):
        """
        The Oanda access token
        
        This field is protected.
        """
        ...

    @property
    def AccountId(self) -> str:
        """
        The Oanda account ID
        
        This field is protected.
        """
        ...

    @AccountId.setter
    def AccountId(self, value: str):
        """
        The Oanda account ID
        
        This field is protected.
        """
        ...

    @property
    def Agent(self) -> str:
        """
        The Oanda agent string
        
        This field is protected.
        """
        ...

    @Agent.setter
    def Agent(self, value: str):
        """
        The Oanda agent string
        
        This field is protected.
        """
        ...

    OandaAgentKey: str = "OANDA-Agent"
    """
    The HTTP header key for Oanda agent
    
    This field is protected.
    """

    OandaAgentDefaultValue: str = "QuantConnect/0.0.0 (LEAN)"
    """The default HTTP header value for Oanda agent"""

    @property
    def IsConnected(self) -> bool:
        """Returns true if we're currently connected to the broker"""
        ...

    def __init__(self, symbolMapper: QuantConnect.Brokerages.Oanda.OandaSymbolMapper, orderProvider: QuantConnect.Securities.IOrderProvider, securityProvider: QuantConnect.Securities.ISecurityProvider, aggregator: QuantConnect.Data.IDataAggregator, environment: QuantConnect.Brokerages.Oanda.Environment, accessToken: str, accountId: str, agent: str) -> None:
        """
        Initializes a new instance of the OandaRestApiBase class.
        
        This method is protected.
        
        :param symbolMapper: The symbol mapper.
        :param orderProvider: The order provider.
        :param securityProvider: The holdings provider.
        :param aggregator: Consolidate ticks
        :param environment: The Oanda environment (Trade or Practice)
        :param accessToken: The Oanda access token (can be the user's personal access token or the access token obtained with OAuth by QC on behalf of the user)
        :param accountId: The account identifier.
        :param agent: The Oanda agent string
        """
        ...

    def Connect(self) -> None:
        """Connects the client to the broker's remote servers"""
        ...

    def Disconnect(self) -> None:
        """Disconnects the client from the broker's remote servers"""
        ...

    def Dispose(self) -> None:
        """Dispose of the brokerage instance"""
        ...

    def DownloadQuoteBars(self, symbol: typing.Union[QuantConnect.Symbol, str], startTimeUtc: typing.Union[datetime.datetime, datetime.date], endTimeUtc: typing.Union[datetime.datetime, datetime.date], resolution: QuantConnect.Resolution, requestedTimeZone: typing.Any) -> System.Collections.Generic.IEnumerable[QuantConnect.Data.Market.QuoteBar]:
        """
        Downloads a list of QuoteBars at the requested resolution
        
        :param symbol: The symbol
        :param startTimeUtc: The starting time (UTC)
        :param endTimeUtc: The ending time (UTC)
        :param resolution: The requested resolution
        :param requestedTimeZone: The requested timezone for the data
        :returns: The list of bars.
        """
        ...

    def DownloadTradeBars(self, symbol: typing.Union[QuantConnect.Symbol, str], startTimeUtc: typing.Union[datetime.datetime, datetime.date], endTimeUtc: typing.Union[datetime.datetime, datetime.date], resolution: QuantConnect.Resolution, requestedTimeZone: typing.Any) -> System.Collections.Generic.IEnumerable[QuantConnect.Data.Market.TradeBar]:
        """
        Downloads a list of TradeBars at the requested resolution
        
        :param symbol: The symbol
        :param startTimeUtc: The starting time (UTC)
        :param endTimeUtc: The ending time (UTC)
        :param resolution: The requested resolution
        :param requestedTimeZone: The requested timezone for the data
        :returns: The list of bars.
        """
        ...

    def EmitTick(self, tick: QuantConnect.Data.Market.Tick) -> None:
        """
        Emit ticks
        
        This method is protected.
        
        :param tick: The new tick to emit
        """
        ...

    def GetAccountBaseCurrency(self) -> str:
        """Gets the account base currency"""
        ...

    def GetInstrumentList(self) -> System.Collections.Generic.List[str]:
        """Gets the list of available tradable instruments/products from Oanda"""
        ...

    def GetRates(self, instruments: System.Collections.Generic.List[str]) -> System.Collections.Generic.Dictionary[str, QuantConnect.Data.Market.Tick]:
        """
        Retrieves the current rate for each of a list of instruments
        
        :param instruments: the list of instruments to check
        :returns: Dictionary containing the current quotes for each instrument.
        """
        ...

    def SetJob(self, job: QuantConnect.Packets.LiveNodePacket) -> None:
        """
        Sets the job we're subscribing for
        
        :param job: Job we're subscribing for
        """
        ...

    def StartPricingStream(self, instruments: System.Collections.Generic.List[str]) -> None:
        """Starts streaming prices for a list of instruments"""
        ...

    def StartTransactionStream(self) -> None:
        """Starts streaming transactions for the active account"""
        ...

    def StopPricingStream(self) -> None:
        """Stops streaming prices for all instruments"""
        ...

    def StopTransactionStream(self) -> None:
        """Stops streaming transactions for the active account"""
        ...

    def Subscribe(self, dataConfig: QuantConnect.Data.SubscriptionDataConfig, newDataAvailableHandler: typing.Callable[[System.Object, System.EventArgs], None]) -> System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]:
        """
        Subscribe to the specified configuration
        
        :param dataConfig: defines the parameters to subscribe to a data feed
        :param newDataAvailableHandler: handler to be fired on new data available
        :returns: The new enumerator for this subscription request.
        """
        ...

    def SubscribeSymbols(self, symbolsToSubscribe: System.Collections.Generic.IEnumerable[QuantConnect.Symbol]) -> None:
        """
        Subscribes to the requested symbols (using a single streaming session)
        
        This method is protected.
        
        :param symbolsToSubscribe: The list of symbols to subscribe
        """
        ...

    def Unsubscribe(self, dataConfig: QuantConnect.Data.SubscriptionDataConfig) -> None:
        """
        Removes the specified configuration
        
        :param dataConfig: Subscription config to be removed
        """
        ...


class OandaRestApiV20(QuantConnect.Brokerages.Oanda.OandaRestApiBase):
    """Oanda REST API v20 implementation"""

    def __init__(self, symbolMapper: QuantConnect.Brokerages.Oanda.OandaSymbolMapper, orderProvider: QuantConnect.Securities.IOrderProvider, securityProvider: QuantConnect.Securities.ISecurityProvider, aggregator: QuantConnect.Data.IDataAggregator, environment: QuantConnect.Brokerages.Oanda.Environment, accessToken: str, accountId: str, agent: str) -> None:
        """
        Initializes a new instance of the OandaRestApiV20 class.
        
        :param symbolMapper: The symbol mapper.
        :param orderProvider: The order provider.
        :param securityProvider: The holdings provider.
        :param aggregator: Consolidate ticks.
        :param environment: The Oanda environment (Trade or Practice)
        :param accessToken: The Oanda access token (can be the user's personal access token or the access token obtained with OAuth by QC on behalf of the user)
        :param accountId: The account identifier.
        :param agent: The Oanda agent string
        """
        ...

    def CancelOrder(self, order: QuantConnect.Orders.Order) -> bool:
        """
        Cancels the order with the specified ID
        
        :param order: The order to cancel
        :returns: True if the request was made for the order to be canceled, false otherwise.
        """
        ...

    def DownloadQuoteBars(self, symbol: typing.Union[QuantConnect.Symbol, str], startTimeUtc: typing.Union[datetime.datetime, datetime.date], endTimeUtc: typing.Union[datetime.datetime, datetime.date], resolution: QuantConnect.Resolution, requestedTimeZone: typing.Any) -> System.Collections.Generic.IEnumerable[QuantConnect.Data.Market.QuoteBar]:
        """
        Downloads a list of QuoteBars at the requested resolution
        
        :param symbol: The symbol
        :param startTimeUtc: The starting time (UTC)
        :param endTimeUtc: The ending time (UTC)
        :param resolution: The requested resolution
        :param requestedTimeZone: The requested timezone for the data
        :returns: The list of bars.
        """
        ...

    def DownloadTradeBars(self, symbol: typing.Union[QuantConnect.Symbol, str], startTimeUtc: typing.Union[datetime.datetime, datetime.date], endTimeUtc: typing.Union[datetime.datetime, datetime.date], resolution: QuantConnect.Resolution, requestedTimeZone: typing.Any) -> System.Collections.Generic.IEnumerable[QuantConnect.Data.Market.TradeBar]:
        """
        Downloads a list of TradeBars at the requested resolution
        
        :param symbol: The symbol
        :param startTimeUtc: The starting time (UTC)
        :param endTimeUtc: The ending time (UTC)
        :param resolution: The requested resolution
        :param requestedTimeZone: The requested timezone for the data
        :returns: The list of bars.
        """
        ...

    def GetAccountBaseCurrency(self) -> str:
        """Gets the account base currency"""
        ...

    def GetAccountHoldings(self) -> System.Collections.Generic.List[QuantConnect.Holding]:
        """
        Gets all holdings for the account
        
        :returns: The current holdings from the account.
        """
        ...

    def GetCashBalance(self) -> System.Collections.Generic.List[QuantConnect.Securities.CashAmount]:
        """
        Gets the current cash balance for each currency held in the brokerage account
        
        :returns: The current cash balance for each currency available for trading.
        """
        ...

    def GetInstrumentList(self) -> System.Collections.Generic.List[str]:
        """Gets the list of available tradable instruments/products from Oanda"""
        ...

    def GetOpenOrders(self) -> System.Collections.Generic.List[QuantConnect.Orders.Order]:
        """
        Gets all open orders on the account.
        NOTE: The order objects returned do not have QC order IDs.
        
        :returns: The open orders returned from Oanda.
        """
        ...

    def GetRates(self, instruments: System.Collections.Generic.List[str]) -> System.Collections.Generic.Dictionary[str, QuantConnect.Data.Market.Tick]:
        """
        Retrieves the current rate for each of a list of instruments
        
        :param instruments: the list of instruments to check
        :returns: Dictionary containing the current quotes for each instrument.
        """
        ...

    def PlaceOrder(self, order: QuantConnect.Orders.Order) -> bool:
        """
        Places a new order and assigns a new broker ID to the order
        
        :param order: The order to be placed
        :returns: True if the request for a new order has been placed, false otherwise.
        """
        ...

    def StartEventsSession(self) -> typing.Any:
        """
        Initializes a streaming events session which will stream events for the given accounts
        
        :returns: the WebResponse object that can be used to retrieve the events as they stream.
        """
        ...

    def StartPricingStream(self, instruments: System.Collections.Generic.List[str]) -> None:
        """Starts streaming prices for a list of instruments"""
        ...

    def StartRatesSession(self, instruments: System.Collections.Generic.List[str]) -> typing.Any:
        """
        Initializes a streaming rates session with the given instruments on the given account
        
        :param instruments: list of instruments to stream rates for
        :returns: the WebResponse object that can be used to retrieve the rates as they stream.
        """
        ...

    def StartTransactionStream(self) -> None:
        """Starts streaming transactions for the active account"""
        ...

    def StopPricingStream(self) -> None:
        """Stops streaming prices for all instruments"""
        ...

    def StopTransactionStream(self) -> None:
        """Stops streaming transactions for the active account"""
        ...

    def UpdateOrder(self, order: QuantConnect.Orders.Order) -> bool:
        """
        Updates the order with the same id
        
        :param order: The new order information
        :returns: True if the request was made for the order to be updated, false otherwise.
        """
        ...


class Server(System.Enum):
    """Represents the server instance that we will be performing the RESTful call."""

    Account = 0
    """The account"""

    Rates = 1
    """The rates"""

    StreamingRates = 2
    """The streaming rates"""

    StreamingEvents = 3
    """The streaming events"""


class OandaBrokerageFactory(QuantConnect.Brokerages.BrokerageFactory):
    """Provides an implementations of IBrokerageFactory that produces a OandaBrokerage"""

    @property
    def BrokerageData(self) -> System.Collections.Generic.Dictionary[str, str]:
        """Gets the brokerage data required to run the brokerage from configuration/disk"""
        ...

    def __init__(self) -> None:
        """Initializes a new instance of the OandaBrokerageFactory class."""
        ...

    def CreateBrokerage(self, job: QuantConnect.Packets.LiveNodePacket, algorithm: QuantConnect.Interfaces.IAlgorithm) -> QuantConnect.Interfaces.IBrokerage:
        """
        Creates a new IBrokerage instance
        
        :param job: The job packet to create the brokerage for
        :param algorithm: The algorithm instance
        :returns: A new brokerage instance.
        """
        ...

    def Dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def GetBrokerageModel(self, orderProvider: QuantConnect.Securities.IOrderProvider) -> QuantConnect.Brokerages.IBrokerageModel:
        """
        Gets a new instance of the OandaBrokerageModel
        
        :param orderProvider: The order provider
        """
        ...





from typing import overload
import datetime
import typing

import QuantConnect
import QuantConnect.Brokerages
import QuantConnect.Brokerages.Bitfinex
import QuantConnect.Data
import QuantConnect.Data.Market
import QuantConnect.Interfaces
import QuantConnect.Orders
import QuantConnect.Packets
import QuantConnect.Securities
import System
import System.Collections.Concurrent
import System.Collections.Generic


class BitfinexWebSocketChannels(System.Collections.Concurrent.ConcurrentDictionary[int, QuantConnect.Data.Channel]):
    """Contains the channel mappings for a WebSocket connection"""

    def Contains(self, channel: QuantConnect.Data.Channel) -> bool:
        """
        Determines whether the dictionary contains a specific channel.
        
        :param channel: The channel
        :returns: true if the channel was found.
        """
        ...

    def GetChannelId(self, channel: QuantConnect.Data.Channel) -> int:
        """
        Returns the channel id for the given channel.
        
        :param channel: The channel
        :returns: The channel id.
        """
        ...


class BitfinexWebSocketWrapper(QuantConnect.Brokerages.WebSocketClientWrapper):
    """Wrapper class for a Bitfinex websocket connection"""

    @property
    def ConnectionId(self) -> str:
        """The unique Id for the connection"""
        ...

    @property
    def ConnectionHandler(self) -> QuantConnect.Brokerages.IConnectionHandler:
        """The handler for the connection"""
        ...

    def __init__(self, connectionHandler: QuantConnect.Brokerages.IConnectionHandler) -> None:
        """Initializes a new instance of the BitfinexWebSocketWrapper class."""
        ...


class BitfinexBrokerage(QuantConnect.Brokerages.BaseWebsocketsBrokerage, QuantConnect.Interfaces.IDataQueueHandler):
    """Bitfinex Brokerage implementation"""

    @property
    def TickLocker(self) -> System.Object:
        """Locking object for the Ticks list in the data queue handler"""
        ...

    @property
    def UnixEpoch(self) -> datetime.datetime:
        """Unix Epoch"""
        ...

    ApiKeyHeader: str = "bfx-apikey"
    """ApiKey Header"""

    NonceHeader: str = "bfx-nonce"
    """Nonce Header"""

    SignatureHeader: str = "bfx-signature"
    """Signature Header"""

    @property
    def IsConnected(self) -> bool:
        ...

    @overload
    def __init__(self) -> None:
        """Constructor for brokerage"""
        ...

    @overload
    def __init__(self, apiKey: str, apiSecret: str, algorithm: QuantConnect.Interfaces.IAlgorithm, priceProvider: QuantConnect.Interfaces.IPriceProvider, aggregator: QuantConnect.Data.IDataAggregator, job: QuantConnect.Packets.LiveNodePacket) -> None:
        """
        Constructor for brokerage
        
        :param apiKey: api key
        :param apiSecret: api secret
        :param algorithm: the algorithm instance is required to retrieve account type
        :param priceProvider: The price provider for missing FX conversion rates
        :param aggregator: consolidate ticks
        :param job: The live job packet
        """
        ...

    @overload
    def __init__(self, websocket: QuantConnect.Brokerages.IWebSocket, restClient: typing.Any, apiKey: str, apiSecret: str, algorithm: QuantConnect.Interfaces.IAlgorithm, priceProvider: QuantConnect.Interfaces.IPriceProvider, aggregator: QuantConnect.Data.IDataAggregator, job: QuantConnect.Packets.LiveNodePacket) -> None:
        """
        Constructor for brokerage
        
        :param websocket: instance of websockets client
        :param restClient: instance of rest client
        :param apiKey: api key
        :param apiSecret: api secret
        :param algorithm: the algorithm instance is required to retrieve account type
        :param priceProvider: The price provider for missing FX conversion rates
        :param aggregator: consolidate ticks
        :param job: The live job packet
        """
        ...

    def CancelOrder(self, order: QuantConnect.Orders.Order) -> bool:
        """
        Cancels the order with the specified ID
        
        :param order: The order to cancel
        :returns: True if the request was submitted for cancellation, false otherwise.
        """
        ...

    def Disconnect(self) -> None:
        """Closes the websockets connection"""
        ...

    def Dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def GetAccountHoldings(self) -> System.Collections.Generic.List[QuantConnect.Holding]:
        """Gets all open positions"""
        ...

    def GetCashBalance(self) -> System.Collections.Generic.List[QuantConnect.Securities.CashAmount]:
        """Gets the total account cash balance for specified account type"""
        ...

    def GetHistory(self, request: QuantConnect.Data.HistoryRequest) -> System.Collections.Generic.IEnumerable[QuantConnect.Data.BaseData]:
        """
        Gets the history for the requested security
        
        :param request: The historical data request
        :returns: An enumerable of bars covering the span specified in the request.
        """
        ...

    def GetOpenOrders(self) -> System.Collections.Generic.List[QuantConnect.Orders.Order]:
        """Gets all orders not yet closed"""
        ...

    def GetSubscribed(self) -> System.Collections.Generic.IEnumerable[QuantConnect.Symbol]:
        """
        Should be empty. BrokerageMultiWebSocketSubscriptionManager manages each BitfinexWebSocketWrapper individually
        
        This method is protected.
        """
        ...

    def GetTick(self, symbol: typing.Union[QuantConnect.Symbol, str]) -> QuantConnect.Data.Market.Tick:
        """Provides the current best bid and ask"""
        ...

    @overload
    def OnMessage(self, sender: typing.Any, e: QuantConnect.Brokerages.WebSocketMessage) -> None:
        """
        Wss message handler
        
        This method is protected.
        """
        ...

    @overload
    def OnMessage(self, e: QuantConnect.Brokerages.BrokerageMessageEvent) -> None:
        ...

    def PlaceOrder(self, order: QuantConnect.Orders.Order) -> bool:
        """
        Places a new order and assigns a new broker ID to the order
        
        :param order: The order to be placed
        :returns: True if the request for a new order has been placed, false otherwise.
        """
        ...

    def SetJob(self, job: QuantConnect.Packets.LiveNodePacket) -> None:
        ...

    @overload
    def Subscribe(self, symbols: System.Collections.Generic.IEnumerable[QuantConnect.Symbol]) -> bool:
        """
        Should be empty, Bitfinex brokerage manages his public channels including subscribe/unsubscribe/reconnect methods using BrokerageMultiWebSocketSubscriptionManager
        Not used in master
        
        This method is protected.
        """
        ...

    @overload
    def Subscribe(self, dataConfig: QuantConnect.Data.SubscriptionDataConfig, newDataAvailableHandler: typing.Callable[[System.Object, System.EventArgs], None]) -> System.Collections.Generic.IEnumerator[QuantConnect.Data.BaseData]:
        """
        Subscribe to the specified configuration
        
        :param dataConfig: defines the parameters to subscribe to a data feed
        :param newDataAvailableHandler: handler to be fired on new data available
        :returns: The new enumerator for this subscription request.
        """
        ...

    def SubscribeAuth(self) -> None:
        """Subscribes to the authenticated channels (using an single streaming channel)"""
        ...

    def Unsubscribe(self, dataConfig: QuantConnect.Data.SubscriptionDataConfig) -> None:
        """
        Removes the specified configuration
        
        :param dataConfig: Subscription config to be removed
        """
        ...

    def UpdateOrder(self, order: QuantConnect.Orders.Order) -> bool:
        """
        Updates the order with the same id
        
        :param order: The new order information
        :returns: True if the request was made for the order to be updated, false otherwise.
        """
        ...


class BitfinexBrokerageFactory(QuantConnect.Brokerages.BrokerageFactory):
    """Factory method to create Bitfinex Websockets brokerage"""

    @property
    def BrokerageData(self) -> System.Collections.Generic.Dictionary[str, str]:
        """provides brokerage connection data"""
        ...

    def __init__(self) -> None:
        """Factory constructor"""
        ...

    def CreateBrokerage(self, job: QuantConnect.Packets.LiveNodePacket, algorithm: QuantConnect.Interfaces.IAlgorithm) -> QuantConnect.Interfaces.IBrokerage:
        """Create the Brokerage instance"""
        ...

    def Dispose(self) -> None:
        """Not required"""
        ...

    def GetBrokerageModel(self, orderProvider: QuantConnect.Securities.IOrderProvider) -> QuantConnect.Brokerages.IBrokerageModel:
        """
        The brokerage model
        
        :param orderProvider: The order provider
        """
        ...


